@@ -11,7 +11,7 @@
 #  include "GHOST_ImeWin32.hh"
 #  include "GHOST_C-api.h"
 #  include "GHOST_WindowWin32.hh"
-#  include "utfconv.h"
+#  include "utfconv.hh"
 
 /* ISO_639-1 2-Letter Abbreviations. */
 #  define IMELANG_ENGLISH "en"
@@ -250,7 +250,7 @@ void GHOST_ImeWin32::GetCaret(HIMC imm_context, LPARAM lparam, ImeComposition *c
 {
   /**
    * This operation is optional and language-dependent because the caret
-   * style is depended on the language, e.g.:
+   * style is dependent on the language, e.g.:
    *   * Korean IMEs: the caret is a blinking block,
    *     (It contains only one hangul character);
    *   * Chinese IMEs: the caret is a blinking line,
