@@ -14,22 +14,24 @@
 #include "DNA_object_force_types.h"
 #include "DNA_scene_types.h"
 
-#include "BLI_blenlib.h"
 #include "BLI_kdtree.h"
+#include "BLI_math_base_safe.h"
 #include "BLI_math_rotation.h"
 #include "BLI_math_vector.h"
 #include "BLI_rand.h"
+#include "BLI_string.h"
+#include "BLI_string_utf8.h"
 #include "BLI_utildefines.h"
 
 #include "BKE_boids.h"
 #include "BKE_collision.h"
 #include "BKE_effect.h"
 #include "BKE_particle.h"
-#include "BLI_kdopbvh.h"
+#include "BLI_kdopbvh.hh"
 
-#include "BLT_translation.h"
+#include "BLT_translation.hh"
 
-#include "BKE_modifier.h"
+#include "BKE_modifier.hh"
 
 #include "RNA_enum_types.hh"
 
@@ -275,13 +277,13 @@ static bool rule_avoid_collision(BoidRule *rule,
       mul_v3_fl(bbd->wanted_co, (1.0f - t) * val->personal_space * pa->size);
 
       bbd->wanted_speed = sqrtf(t) * len_v3(pa->prev_state.vel);
-      bbd->wanted_speed = MAX2(bbd->wanted_speed, val->min_speed);
+      bbd->wanted_speed = std::max(bbd->wanted_speed, val->min_speed);
 
       return true;
     }
   }
 
-  /* Check boids in own system. */
+  /* Check boids in their own system. */
   if (acbr->options & BRULE_ACOLL_WITH_BOIDS) {
     neighbors = BLI_kdtree_3d_range_search_with_len_squared_cb(bbd->sim->psys->tree,
                                                                pa->prev_state.co,
@@ -689,7 +691,7 @@ static bool rule_fight(BoidRule *rule, BoidBrainData *bbd, BoidValues *val, Part
   int n;
   bool ret = false;
 
-  /* calculate own group strength */
+  /* calculate its own group strength */
   int neighbors = BLI_kdtree_3d_range_search(
       bbd->sim->psys->tree, pa->prev_state.co, &ptn, fbr->distance);
   for (n = 0; n < neighbors; n++) {
@@ -704,7 +706,7 @@ static bool rule_fight(BoidRule *rule, BoidBrainData *bbd, BoidValues *val, Part
   /* add other friendlies and calculate enemy strength and find closest enemy */
   LISTBASE_FOREACH (ParticleTarget *, pt, &bbd->sim->psys->targets) {
     ParticleSystem *epsys = psys_get_target_system(bbd->sim->ob, pt);
-    if (epsys) {
+    if (epsys && epsys->part->boids) {
       epars = epsys->particles;
 
       neighbors = BLI_kdtree_3d_range_search(epsys->tree, pa->prev_state.co, &ptn, fbr->distance);
@@ -961,7 +963,7 @@ void boids_precalc_rules(ParticleSettings *part, float cfra)
         if (flbr->ob && flbr->cfra != cfra) {
           /* save object locations for velocity calculations */
           copy_v3_v3(flbr->oloc, flbr->loc);
-          copy_v3_v3(flbr->loc, flbr->ob->object_to_world[3]);
+          copy_v3_v3(flbr->loc, flbr->ob->object_to_world().location());
           flbr->cfra = cfra;
         }
       }
@@ -1169,17 +1171,18 @@ void boid_brain(BoidBrainData *bbd, int p, ParticleData *pa)
         if (dot_v2v2(cvel, dir) > 0.95f / mul || len <= state->rule_fuzziness) {
           /* try to reach goal at highest point of the parabolic path */
           cur_v = len_v2(pa->prev_state.vel);
-          z_v = sasqrt(-2.0f * bbd->sim->scene->physics_settings.gravity[2] * bbd->wanted_co[2]);
+          z_v = safe_sqrtf(-2.0f * bbd->sim->scene->physics_settings.gravity[2] *
+                           bbd->wanted_co[2]);
           ground_v = len_v2(bbd->wanted_co) *
-                     sasqrt(-0.5f * bbd->sim->scene->physics_settings.gravity[2] /
-                            bbd->wanted_co[2]);
+                     safe_sqrtf(-0.5f * bbd->sim->scene->physics_settings.gravity[2] /
+                                bbd->wanted_co[2]);
 
-          len = sasqrt((ground_v - cur_v) * (ground_v - cur_v) + z_v * z_v);
+          len = safe_sqrtf((ground_v - cur_v) * (ground_v - cur_v) + z_v * z_v);
 
           if (len < val.jump_speed * mul || bbd->part->boids->options & BOID_ALLOW_FLIGHT) {
             jump = 1;
 
-            len = MIN2(len, val.jump_speed);
+            len = std::min(len, val.jump_speed);
 
             copy_v3_v3(jump_v, dir);
             jump_v[2] = z_v;
@@ -1298,7 +1301,7 @@ void boid_body(BoidBrainData *bbd, ParticleData *pa)
       }
 
       /* constrain direction with maximum angular velocity */
-      angle = saacos(dot_v3v3(old_dir, wanted_dir));
+      angle = safe_acosf(dot_v3v3(old_dir, wanted_dir));
       angle = min_ff(angle, val.max_ave);
 
       cross_v3_v3v3(nor, old_dir, wanted_dir);
@@ -1318,10 +1321,10 @@ void boid_body(BoidBrainData *bbd, ParticleData *pa)
     old_speed = len_v3(pa->prev_state.vel);
 
     if (bbd->wanted_speed < old_speed) {
-      new_speed = MAX2(bbd->wanted_speed, old_speed - val.max_acc);
+      new_speed = std::max(bbd->wanted_speed, old_speed - val.max_acc);
     }
     else {
-      new_speed = MIN2(bbd->wanted_speed, old_speed + val.max_acc);
+      new_speed = std::min(bbd->wanted_speed, old_speed + val.max_acc);
     }
 
     /* combine direction and speed */
@@ -1333,18 +1336,18 @@ void boid_body(BoidBrainData *bbd, ParticleData *pa)
       float len2 = dot_v2v2(new_vel, new_vel);
       float root;
 
-      len2 = MAX2(len2, val.min_speed * val.min_speed);
-      root = sasqrt(new_speed * new_speed - len2);
+      len2 = std::max(len2, val.min_speed * val.min_speed);
+      root = safe_sqrtf(new_speed * new_speed - len2);
 
       new_vel[2] = new_vel[2] < 0.0f ? -root : root;
 
       normalize_v2(new_vel);
-      mul_v2_fl(new_vel, sasqrt(len2));
+      mul_v2_fl(new_vel, safe_sqrtf(len2));
     }
 
     /* finally constrain speed to max speed */
     new_speed = normalize_v3(new_vel);
-    mul_v3_fl(new_vel, MIN2(new_speed, val.max_speed));
+    mul_v3_fl(new_vel, std::min(new_speed, val.max_speed));
 
     /* get acceleration from difference of velocities */
     sub_v3_v3v3(acc, new_vel, pa->prev_state.vel);
@@ -1367,7 +1370,7 @@ void boid_body(BoidBrainData *bbd, ParticleData *pa)
   if (ELEM(bpa->data.mode, eBoidMode_OnLand, eBoidMode_Climbing)) {
     float length = normalize_v3(force);
 
-    length = MAX2(0.0f, length - boids->land_stick_force);
+    length = std::max(0.0f, length - boids->land_stick_force);
 
     mul_v3_fl(force, length);
   }
