@@ -11,13 +11,7 @@
  * \see mesh_tessellate.cc for the #Mesh equivalent of this file.
  */
 
-#include "DNA_meshdata_types.h"
-
-#include "MEM_guardedalloc.h"
-
-#include "BLI_alloca.h"
 #include "BLI_heap.h"
-#include "BLI_linklist.h"
 #include "BLI_math_geom.h"
 #include "BLI_math_matrix.h"
 #include "BLI_memarena.h"
@@ -25,8 +19,9 @@
 #include "BLI_polyfill_2d_beautify.h"
 #include "BLI_task.h"
 
-#include "bmesh.h"
-#include "bmesh_tools.h"
+#include "bmesh.hh"
+
+using blender::MutableSpan;
 
 /**
  * On systems with 32+ cores,
@@ -44,12 +39,12 @@
 /**
  * \param face_normal: This will be optimized out as a constant.
  */
-BLI_INLINE void bmesh_calc_tessellation_for_face_impl(BMLoop *(*looptris)[3],
+BLI_INLINE void bmesh_calc_tessellation_for_face_impl(std::array<BMLoop *, 3> *looptris,
                                                       BMFace *efa,
                                                       MemArena **pf_arena_p,
                                                       const bool face_normal)
 {
-#ifdef DEBUG
+#ifndef NDEBUG
   /* The face normal is used for projecting faces into 2D space for tessellation.
    * Invalid normals may result in invalid tessellation.
    * Either `face_normal` should be true or normals should be updated first. */
@@ -60,7 +55,7 @@ BLI_INLINE void bmesh_calc_tessellation_for_face_impl(BMLoop *(*looptris)[3],
     case 3: {
       /* `0 1 2` -> `0 1 2` */
       BMLoop *l;
-      BMLoop **l_ptr = looptris[0];
+      BMLoop **l_ptr = looptris[0].data();
       l_ptr[0] = l = BM_FACE_FIRST_LOOP(efa);
       l_ptr[1] = l = l->next;
       l_ptr[2] = l->next;
@@ -72,8 +67,8 @@ BLI_INLINE void bmesh_calc_tessellation_for_face_impl(BMLoop *(*looptris)[3],
     case 4: {
       /* `0 1 2 3` -> (`0 1 2`, `0 2 3`) */
       BMLoop *l;
-      BMLoop **l_ptr_a = looptris[0];
-      BMLoop **l_ptr_b = looptris[1];
+      BMLoop **l_ptr_a = looptris[0].data();
+      BMLoop **l_ptr_b = looptris[1].data();
       (l_ptr_a[0] = l_ptr_b[0] = l = BM_FACE_FIRST_LOOP(efa));
       (l_ptr_a[1] = l = l->next);
       (l_ptr_a[2] = l_ptr_b[1] = l = l->next);
@@ -130,7 +125,7 @@ BLI_INLINE void bmesh_calc_tessellation_for_face_impl(BMLoop *(*looptris)[3],
       BLI_polyfill_calc_arena(projverts, efa->len, 1, tris, pf_arena);
 
       for (i = 0; i < tris_len; i++) {
-        BMLoop **l_ptr = looptris[i];
+        BMLoop **l_ptr = looptris[i].data();
         uint *tri = tris[i];
 
         l_ptr[0] = l_arr[tri[0]];
@@ -144,14 +139,14 @@ BLI_INLINE void bmesh_calc_tessellation_for_face_impl(BMLoop *(*looptris)[3],
   }
 }
 
-static void bmesh_calc_tessellation_for_face(BMLoop *(*looptris)[3],
+static void bmesh_calc_tessellation_for_face(std::array<BMLoop *, 3> *looptris,
                                              BMFace *efa,
                                              MemArena **pf_arena_p)
 {
   bmesh_calc_tessellation_for_face_impl(looptris, efa, pf_arena_p, false);
 }
 
-static void bmesh_calc_tessellation_for_face_with_normal(BMLoop *(*looptris)[3],
+static void bmesh_calc_tessellation_for_face_with_normal(std::array<BMLoop *, 3> *looptris,
                                                          BMFace *efa,
                                                          MemArena **pf_arena_p)
 {
@@ -164,9 +159,8 @@ static void bmesh_calc_tessellation_for_face_with_normal(BMLoop *(*looptris)[3],
  *
  * \note \a looptris Must be pre-allocated to at least the size of given by: poly_to_tri_count
  */
-static void bm_mesh_calc_tessellation__single_threaded(BMesh *bm,
-                                                       BMLoop *(*looptris)[3],
-                                                       const char face_normals)
+static void bm_mesh_calc_tessellation__single_threaded(
+    BMesh *bm, MutableSpan<std::array<BMLoop *, 3>> looptris, const char face_normals)
 {
 #ifndef NDEBUG
   const int looptris_tot = poly_to_tri_count(bm->totface, bm->totloop);
@@ -182,14 +176,14 @@ static void bm_mesh_calc_tessellation__single_threaded(BMesh *bm,
     BM_ITER_MESH (efa, &iter, bm, BM_FACES_OF_MESH) {
       BLI_assert(efa->len >= 3);
       BM_face_calc_normal(efa, efa->no);
-      bmesh_calc_tessellation_for_face_with_normal(looptris + i, efa, &pf_arena);
+      bmesh_calc_tessellation_for_face_with_normal(looptris.data() + i, efa, &pf_arena);
       i += efa->len - 2;
     }
   }
   else {
     BM_ITER_MESH (efa, &iter, bm, BM_FACES_OF_MESH) {
       BLI_assert(efa->len >= 3);
-      bmesh_calc_tessellation_for_face(looptris + i, efa, &pf_arena);
+      bmesh_calc_tessellation_for_face(looptris.data() + i, efa, &pf_arena);
       i += efa->len - 2;
     }
   }
@@ -211,7 +205,7 @@ static void bmesh_calc_tessellation_for_face_fn(void *__restrict userdata,
                                                 const TaskParallelTLS *__restrict tls)
 {
   TessellationUserTLS *tls_data = static_cast<TessellationUserTLS *>(tls->userdata_chunk);
-  BMLoop *(*looptris)[3] = static_cast<BMLoop *(*)[3]>(userdata);
+  std::array<BMLoop *, 3> *looptris = static_cast<std::array<BMLoop *, 3> *>(userdata);
   BMFace *f = (BMFace *)mp_f;
   BMLoop *l = BM_FACE_FIRST_LOOP(f);
   const int offset = BM_elem_index_get(l) - (BM_elem_index_get(f) * 2);
@@ -223,7 +217,7 @@ static void bmesh_calc_tessellation_for_face_with_normals_fn(void *__restrict us
                                                              const TaskParallelTLS *__restrict tls)
 {
   TessellationUserTLS *tls_data = static_cast<TessellationUserTLS *>(tls->userdata_chunk);
-  BMLoop *(*looptris)[3] = static_cast<BMLoop *(*)[3]>(userdata);
+  std::array<BMLoop *, 3> *looptris = static_cast<std::array<BMLoop *, 3> *>(userdata);
   BMFace *f = (BMFace *)mp_f;
   BMLoop *l = BM_FACE_FIRST_LOOP(f);
   const int offset = BM_elem_index_get(l) - (BM_elem_index_get(f) * 2);
@@ -239,9 +233,8 @@ static void bmesh_calc_tessellation_for_face_free_fn(const void *__restrict /*us
   }
 }
 
-static void bm_mesh_calc_tessellation__multi_threaded(BMesh *bm,
-                                                      BMLoop *(*looptris)[3],
-                                                      const char face_normals)
+static void bm_mesh_calc_tessellation__multi_threaded(
+    BMesh *bm, MutableSpan<std::array<BMLoop *, 3>> looptris, const char face_normals)
 {
   BM_mesh_elem_index_ensure(bm, BM_LOOP | BM_FACE);
 
@@ -255,12 +248,12 @@ static void bm_mesh_calc_tessellation__multi_threaded(BMesh *bm,
                    BM_FACES_OF_MESH,
                    face_normals ? bmesh_calc_tessellation_for_face_with_normals_fn :
                                   bmesh_calc_tessellation_for_face_fn,
-                   looptris,
+                   looptris.data(),
                    &settings);
 }
 
 void BM_mesh_calc_tessellation_ex(BMesh *bm,
-                                  BMLoop *(*looptris)[3],
+                                  MutableSpan<std::array<BMLoop *, 3>> looptris,
                                   const BMeshCalcTessellation_Params *params)
 {
   if (bm->totface < BM_FACE_TESSELLATE_THREADED_LIMIT) {
@@ -271,7 +264,7 @@ void BM_mesh_calc_tessellation_ex(BMesh *bm,
   }
 }
 
-void BM_mesh_calc_tessellation(BMesh *bm, BMLoop *(*looptris)[3])
+void BM_mesh_calc_tessellation(BMesh *bm, MutableSpan<std::array<BMLoop *, 3>> looptris)
 {
   BMeshCalcTessellation_Params params{};
   params.face_normals = false;
@@ -286,7 +279,7 @@ void BM_mesh_calc_tessellation(BMesh *bm, BMLoop *(*looptris)[3])
 
 struct PartialTessellationUserData {
   BMFace **faces;
-  BMLoop *(*looptris)[3];
+  MutableSpan<std::array<BMLoop *, 3>> looptris;
 };
 
 struct PartialTessellationUserTLS {
@@ -303,7 +296,7 @@ static void bmesh_calc_tessellation_for_face_partial_fn(void *__restrict userdat
   BMFace *f = data->faces[index];
   BMLoop *l = BM_FACE_FIRST_LOOP(f);
   const int offset = BM_elem_index_get(l) - (BM_elem_index_get(f) * 2);
-  bmesh_calc_tessellation_for_face(data->looptris + offset, f, &tls_data->pf_arena);
+  bmesh_calc_tessellation_for_face(data->looptris.data() + offset, f, &tls_data->pf_arena);
 }
 
 static void bmesh_calc_tessellation_for_face_partial_with_normals_fn(
@@ -315,7 +308,8 @@ static void bmesh_calc_tessellation_for_face_partial_with_normals_fn(
   BMFace *f = data->faces[index];
   BMLoop *l = BM_FACE_FIRST_LOOP(f);
   const int offset = BM_elem_index_get(l) - (BM_elem_index_get(f) * 2);
-  bmesh_calc_tessellation_for_face_with_normal(data->looptris + offset, f, &tls_data->pf_arena);
+  bmesh_calc_tessellation_for_face_with_normal(
+      data->looptris.data() + offset, f, &tls_data->pf_arena);
 }
 
 static void bmesh_calc_tessellation_for_face_partial_free_fn(const void *__restrict /*userdata*/,
@@ -328,7 +322,7 @@ static void bmesh_calc_tessellation_for_face_partial_free_fn(const void *__restr
 }
 
 static void bm_mesh_calc_tessellation_with_partial__multi_threaded(
-    BMLoop *(*looptris)[3],
+    MutableSpan<std::array<BMLoop *, 3>> looptris,
     const BMPartialUpdate *bmpinfo,
     const BMeshCalcTessellation_Params *params)
 {
@@ -357,7 +351,7 @@ static void bm_mesh_calc_tessellation_with_partial__multi_threaded(
 }
 
 static void bm_mesh_calc_tessellation_with_partial__single_threaded(
-    BMLoop *(*looptris)[3],
+    MutableSpan<std::array<BMLoop *, 3>> looptris,
     const BMPartialUpdate *bmpinfo,
     const BMeshCalcTessellation_Params *params)
 {
@@ -371,7 +365,7 @@ static void bm_mesh_calc_tessellation_with_partial__single_threaded(
       BMFace *f = faces[index];
       BMLoop *l = BM_FACE_FIRST_LOOP(f);
       const int offset = BM_elem_index_get(l) - (BM_elem_index_get(f) * 2);
-      bmesh_calc_tessellation_for_face_with_normal(looptris + offset, f, &pf_arena);
+      bmesh_calc_tessellation_for_face_with_normal(looptris.data() + offset, f, &pf_arena);
     }
   }
   else {
@@ -379,7 +373,7 @@ static void bm_mesh_calc_tessellation_with_partial__single_threaded(
       BMFace *f = faces[index];
       BMLoop *l = BM_FACE_FIRST_LOOP(f);
       const int offset = BM_elem_index_get(l) - (BM_elem_index_get(f) * 2);
-      bmesh_calc_tessellation_for_face(looptris + offset, f, &pf_arena);
+      bmesh_calc_tessellation_for_face(looptris.data() + offset, f, &pf_arena);
     }
   }
 
@@ -389,7 +383,7 @@ static void bm_mesh_calc_tessellation_with_partial__single_threaded(
 }
 
 void BM_mesh_calc_tessellation_with_partial_ex(BMesh *bm,
-                                               BMLoop *(*looptris)[3],
+                                               MutableSpan<std::array<BMLoop *, 3>> looptris,
                                                const BMPartialUpdate *bmpinfo,
                                                const BMeshCalcTessellation_Params *params)
 {
@@ -410,7 +404,7 @@ void BM_mesh_calc_tessellation_with_partial_ex(BMesh *bm,
 }
 
 void BM_mesh_calc_tessellation_with_partial(BMesh *bm,
-                                            BMLoop *(*looptris)[3],
+                                            MutableSpan<std::array<BMLoop *, 3>> looptris,
                                             const BMPartialUpdate *bmpinfo)
 {
   BM_mesh_calc_tessellation_with_partial_ex(bm, looptris, bmpinfo, nullptr);
@@ -424,7 +418,7 @@ void BM_mesh_calc_tessellation_with_partial(BMesh *bm,
  * Avoid degenerate triangles.
  * \{ */
 
-static int bmesh_calc_tessellation_for_face_beauty(BMLoop *(*looptris)[3],
+static int bmesh_calc_tessellation_for_face_beauty(std::array<BMLoop *, 3> *looptris,
                                                    BMFace *efa,
                                                    MemArena **pf_arena_p,
                                                    Heap **pf_heap_p)
@@ -432,7 +426,7 @@ static int bmesh_calc_tessellation_for_face_beauty(BMLoop *(*looptris)[3],
   switch (efa->len) {
     case 3: {
       BMLoop *l;
-      BMLoop **l_ptr = looptris[0];
+      BMLoop **l_ptr = looptris[0].data();
       l_ptr[0] = l = BM_FACE_FIRST_LOOP(efa);
       l_ptr[1] = l = l->next;
       l_ptr[2] = l->next;
@@ -464,8 +458,8 @@ static int bmesh_calc_tessellation_for_face_beauty(BMLoop *(*looptris)[3],
                                 v_quad[0], v_quad[1], v_quad[2], v_quad[3]) < 0.0f;
 #endif
 
-      BMLoop **l_ptr_a = looptris[0];
-      BMLoop **l_ptr_b = looptris[1];
+      BMLoop **l_ptr_a = looptris[0].data();
+      BMLoop **l_ptr_b = looptris[1].data();
       if (split_13) {
         l_ptr_a[0] = l_v1;
         l_ptr_a[1] = l_v2;
@@ -523,7 +517,7 @@ static int bmesh_calc_tessellation_for_face_beauty(BMLoop *(*looptris)[3],
       BLI_polyfill_beautify(projverts, efa->len, tris, pf_arena, pf_heap);
 
       for (i = 0; i < tris_len; i++) {
-        BMLoop **l_ptr = looptris[i];
+        BMLoop **l_ptr = looptris[i].data();
         uint *tri = tris[i];
 
         l_ptr[0] = l_arr[tri[0]];
@@ -538,7 +532,7 @@ static int bmesh_calc_tessellation_for_face_beauty(BMLoop *(*looptris)[3],
   }
 }
 
-void BM_mesh_calc_tessellation_beauty(BMesh *bm, BMLoop *(*looptris)[3])
+void BM_mesh_calc_tessellation_beauty(BMesh *bm, MutableSpan<std::array<BMLoop *, 3>> looptris)
 {
 #ifndef NDEBUG
   const int looptris_tot = poly_to_tri_count(bm->totface, bm->totloop);
@@ -555,7 +549,7 @@ void BM_mesh_calc_tessellation_beauty(BMesh *bm, BMLoop *(*looptris)[3])
 
   BM_ITER_MESH (efa, &iter, bm, BM_FACES_OF_MESH) {
     BLI_assert(efa->len >= 3);
-    i += bmesh_calc_tessellation_for_face_beauty(looptris + i, efa, &pf_arena, &pf_heap);
+    i += bmesh_calc_tessellation_for_face_beauty(looptris.data() + i, efa, &pf_arena, &pf_heap);
   }
 
   if (pf_arena) {
