@@ -6,14 +6,14 @@
  * \ingroup draw
  */
 
-#include "GPU_batch.h"
-#include "GPU_capabilities.h"
-#include "GPU_compute.h"
-#include "GPU_debug.h"
+#include "GPU_batch.hh"
+#include "GPU_capabilities.hh"
+#include "GPU_compute.hh"
+#include "GPU_debug.hh"
 
 #include "draw_command.hh"
 #include "draw_pass.hh"
-#include "draw_shader.h"
+#include "draw_shader.hh"
 #include "draw_view.hh"
 
 #include <bitset>
@@ -21,6 +21,24 @@
 
 namespace blender::draw::command {
 
+static gpu::Batch *procedural_batch_get(GPUPrimType primitive)
+{
+  switch (primitive) {
+    case GPU_PRIM_POINTS:
+      return drw_cache_procedural_points_get();
+    case GPU_PRIM_LINES:
+      return drw_cache_procedural_lines_get();
+    case GPU_PRIM_TRIS:
+      return drw_cache_procedural_triangles_get();
+    case GPU_PRIM_TRI_STRIP:
+      return drw_cache_procedural_triangle_strips_get();
+    default:
+      /* Add new one as needed. */
+      BLI_assert_unreachable();
+      return nullptr;
+  }
+}
+
 /* -------------------------------------------------------------------- */
 /** \name Commands Execution
  * \{ */
@@ -37,6 +55,26 @@ void FramebufferBind::execute() const
   GPU_framebuffer_bind(*framebuffer);
 }
 
+void SubPassTransition::execute() const
+{
+  /* TODO(fclem): Require framebuffer bind to always be part of the pass so that we can track it
+   * inside RecordingState. */
+  GPUFrameBuffer *framebuffer = GPU_framebuffer_active_get();
+  /* Unpack to the real enum type. */
+  const GPUAttachmentState states[9] = {
+      GPUAttachmentState(depth_state),
+      GPUAttachmentState(color_states[0]),
+      GPUAttachmentState(color_states[1]),
+      GPUAttachmentState(color_states[2]),
+      GPUAttachmentState(color_states[3]),
+      GPUAttachmentState(color_states[4]),
+      GPUAttachmentState(color_states[5]),
+      GPUAttachmentState(color_states[6]),
+      GPUAttachmentState(color_states[7]),
+  };
+  GPU_framebuffer_subpass_transition_array(framebuffer, states, ARRAY_SIZE(states));
+}
+
 void ResourceBind::execute() const
 {
   if (slot == -1) {
@@ -91,6 +129,39 @@ void PushConstant::execute(RecordingState &state) const
   }
 }
 
+void SpecializeConstant::execute() const
+{
+  /* All specialization constants should exist as they are not optimized out like uniforms. */
+  BLI_assert(location != -1);
+
+  switch (type) {
+    case SpecializeConstant::Type::IntValue:
+      GPU_shader_constant_int_ex(shader, location, int_value);
+      break;
+    case SpecializeConstant::Type::IntReference:
+      GPU_shader_constant_int_ex(shader, location, *int_ref);
+      break;
+    case SpecializeConstant::Type::UintValue:
+      GPU_shader_constant_uint_ex(shader, location, uint_value);
+      break;
+    case SpecializeConstant::Type::UintReference:
+      GPU_shader_constant_uint_ex(shader, location, *uint_ref);
+      break;
+    case SpecializeConstant::Type::FloatValue:
+      GPU_shader_constant_float_ex(shader, location, float_value);
+      break;
+    case SpecializeConstant::Type::FloatReference:
+      GPU_shader_constant_float_ex(shader, location, *float_ref);
+      break;
+    case SpecializeConstant::Type::BoolValue:
+      GPU_shader_constant_bool_ex(shader, location, bool_value);
+      break;
+    case SpecializeConstant::Type::BoolReference:
+      GPU_shader_constant_bool_ex(shader, location, *bool_ref);
+      break;
+  }
+}
+
 void Draw::execute(RecordingState &state) const
 {
   state.front_facing_set(handle.has_inverted_handedness());
@@ -99,8 +170,39 @@ void Draw::execute(RecordingState &state) const
     GPU_batch_resource_id_buf_set(batch, state.resource_id_buf);
   }
 
-  GPU_batch_set_shader(batch, state.shader);
-  GPU_batch_draw_advanced(batch, vertex_first, vertex_len, 0, instance_len);
+  /* Use same logic as in `finalize_commands`. */
+  uint instance_first = 0;
+  if (handle.raw > 0) {
+    instance_first = state.instance_offset;
+    state.instance_offset += instance_len;
+  }
+
+  if (is_primitive_expansion()) {
+    /* Expanded drawcall. */
+    IndexRange expanded_range = GPU_batch_draw_expanded_parameter_get(
+        batch->prim_type,
+        GPUPrimType(expand_prim_type),
+        vertex_len,
+        vertex_first,
+        expand_prim_len);
+
+    if (expanded_range.is_empty()) {
+      /* Nothing to draw, and can lead to asserts in GPU_batch_bind_as_resources. */
+      return;
+    }
+
+    GPU_batch_bind_as_resources(batch, state.shader);
+
+    gpu::Batch *gpu_batch = procedural_batch_get(GPUPrimType(expand_prim_type));
+    GPU_batch_set_shader(gpu_batch, state.shader);
+    GPU_batch_draw_advanced(
+        gpu_batch, expanded_range.start(), expanded_range.size(), instance_first, instance_len);
+  }
+  else {
+    /* Regular drawcall. */
+    GPU_batch_set_shader(batch, state.shader);
+    GPU_batch_draw_advanced(batch, vertex_first, vertex_len, instance_first, instance_len);
+  }
 }
 
 void DrawMulti::execute(RecordingState &state) const
@@ -113,11 +215,19 @@ void DrawMulti::execute(RecordingState &state) const
     const DrawGroup &group = groups[group_index];
 
     if (group.vertex_len > 0) {
-      if (GPU_shader_draw_parameters_support() == false) {
-        GPU_batch_resource_id_buf_set(group.gpu_batch, state.resource_id_buf);
+      gpu::Batch *batch = group.desc.gpu_batch;
+
+      if (GPUPrimType(group.desc.expand_prim_type) != GPU_PRIM_NONE) {
+        /* Bind original batch as resource and use a procedural batch to issue the draw-call. */
+        GPU_batch_bind_as_resources(group.desc.gpu_batch, state.shader);
+        batch = procedural_batch_get(GPUPrimType(group.desc.expand_prim_type));
       }
 
-      GPU_batch_set_shader(group.gpu_batch, state.shader);
+      if (GPU_shader_draw_parameters_support() == false) {
+        GPU_batch_resource_id_buf_set(batch, state.resource_id_buf);
+      }
+
+      GPU_batch_set_shader(batch, state.shader);
 
       constexpr intptr_t stride = sizeof(DrawCommand);
       /* We have 2 indirect command reserved per draw group. */
@@ -126,12 +236,12 @@ void DrawMulti::execute(RecordingState &state) const
       /* Draw negatively scaled geometry first. */
       if (group.len - group.front_facing_len > 0) {
         state.front_facing_set(true);
-        GPU_batch_draw_indirect(group.gpu_batch, indirect_buf, offset);
+        GPU_batch_draw_indirect(batch, indirect_buf, offset);
       }
 
       if (group.front_facing_len > 0) {
         state.front_facing_set(false);
-        GPU_batch_draw_indirect(group.gpu_batch, indirect_buf, offset + stride);
+        GPU_batch_draw_indirect(batch, indirect_buf, offset + stride);
       }
     }
 
@@ -180,12 +290,6 @@ void ClearMulti::execute() const
 
 void StateSet::execute(RecordingState &recording_state) const
 {
-  /**
-   * Does not support locked state for the moment and never should.
-   * Better implement a less hacky selection!
-   */
-  BLI_assert(DST.state_lock == 0);
-
   bool state_changed = assign_if_different(recording_state.pipeline_state, new_state);
   bool clip_changed = assign_if_different(recording_state.clip_plane_count, clip_plane_count);
 
@@ -193,10 +297,6 @@ void StateSet::execute(RecordingState &recording_state) const
     return;
   }
 
-  /* Keep old API working. Keep the state tracking in sync. */
-  /* TODO(fclem): Move at the end of a pass. */
-  DST.state = new_state;
-
   GPU_state_set(to_write_mask(new_state),
                 to_blend(new_state),
                 to_face_cull_test(new_state),
@@ -234,6 +334,25 @@ void StateSet::execute(RecordingState &recording_state) const
   }
 }
 
+/* Set state of the GPU module manually. */
+void StateSet::set(DRWState state)
+{
+  RecordingState recording_state;
+  StateSet{state, 0}.execute(recording_state);
+
+  /* This function is used for cleaning the state for the viewport drawing.
+   * Make sure to reset textures resources to avoid feedback loop when rendering (see #131652). */
+  GPU_texture_unbind_all();
+  GPU_texture_image_unbind_all();
+  GPU_uniformbuf_debug_unbind_all();
+  GPU_storagebuf_debug_unbind_all();
+
+  /* Remained of legacy draw manager. Kept it to avoid regression, but might become unneeded. */
+  GPU_point_size(5);
+  GPU_line_smooth(false);
+  GPU_line_width(0.0f);
+}
+
 void StencilSet::execute() const
 {
   GPU_stencil_write_mask_set(write_mask);
@@ -258,6 +377,26 @@ std::string FramebufferBind::serialize() const
          (*framebuffer == nullptr ? "nullptr" : GPU_framebuffer_get_name(*framebuffer)) + ")";
 }
 
+std::string SubPassTransition::serialize() const
+{
+  auto to_str = [](GPUAttachmentState state) {
+    return (state != GPU_ATTACHMENT_IGNORE) ?
+               ((state == GPU_ATTACHMENT_WRITE) ? "write" : "read") :
+               "ignore";
+  };
+
+  return std::string(".subpass_transition(\n") +
+         "depth=" + to_str(GPUAttachmentState(depth_state)) + ",\n" +
+         "color0=" + to_str(GPUAttachmentState(color_states[0])) + ",\n" +
+         "color1=" + to_str(GPUAttachmentState(color_states[1])) + ",\n" +
+         "color2=" + to_str(GPUAttachmentState(color_states[2])) + ",\n" +
+         "color3=" + to_str(GPUAttachmentState(color_states[3])) + ",\n" +
+         "color4=" + to_str(GPUAttachmentState(color_states[4])) + ",\n" +
+         "color5=" + to_str(GPUAttachmentState(color_states[5])) + ",\n" +
+         "color6=" + to_str(GPUAttachmentState(color_states[6])) + ",\n" +
+         "color7=" + to_str(GPUAttachmentState(color_states[7])) + "\n)";
+}
+
 std::string ResourceBind::serialize() const
 {
   switch (type) {
@@ -384,9 +523,42 @@ std::string PushConstant::serialize() const
   return std::string(".push_constant(") + std::to_string(location) + ", data=" + ss.str() + ")";
 }
 
+std::string SpecializeConstant::serialize() const
+{
+  std::stringstream ss;
+  switch (type) {
+    case Type::IntValue:
+      ss << int_value;
+      break;
+    case Type::UintValue:
+      ss << uint_value;
+      break;
+    case Type::FloatValue:
+      ss << float_value;
+      break;
+    case Type::BoolValue:
+      ss << bool_value;
+      break;
+    case Type::IntReference:
+      ss << *int_ref;
+      break;
+    case Type::UintReference:
+      ss << *uint_ref;
+      break;
+    case Type::FloatReference:
+      ss << *float_ref;
+      break;
+    case Type::BoolReference:
+      ss << *bool_ref;
+      break;
+  }
+  return std::string(".specialize_constant(") + std::to_string(location) + ", data=" + ss.str() +
+         ")";
+}
+
 std::string Draw::serialize() const
 {
-  std::string inst_len = (instance_len == uint(-1)) ? "from_batch" : std::to_string(instance_len);
+  std::string inst_len = std::to_string(instance_len);
   std::string vert_len = (vertex_len == uint(-1)) ? "from_batch" : std::to_string(vertex_len);
   std::string vert_first = (vertex_first == uint(-1)) ? "from_batch" :
                                                         std::to_string(vertex_first);
@@ -395,7 +567,7 @@ std::string Draw::serialize() const
          ")";
 }
 
-std::string DrawMulti::serialize(std::string line_prefix) const
+std::string DrawMulti::serialize(const std::string &line_prefix) const
 {
   DrawMultiBuf::DrawGroupBuf &groups = multi_draw_buf->group_buf_;
 
@@ -406,14 +578,14 @@ std::string DrawMulti::serialize(std::string line_prefix) const
   std::sort(
       prototypes.begin(), prototypes.end(), [](const DrawPrototype &a, const DrawPrototype &b) {
         return (a.group_id < b.group_id) ||
-               (a.group_id == b.group_id && a.resource_handle > b.resource_handle);
+               (a.group_id == b.group_id && a.res_handle > b.res_handle);
       });
 
   /* Compute prefix sum to have correct offsets. */
   uint prefix_sum = 0u;
   for (DrawGroup &group : groups) {
     group.start = prefix_sum;
-    prefix_sum += group.front_proto_len + group.back_proto_len;
+    prefix_sum += group.front_facing_counter + group.back_facing_counter;
   }
 
   std::stringstream ss;
@@ -427,22 +599,22 @@ std::string DrawMulti::serialize(std::string line_prefix) const
 
     intptr_t offset = grp.start;
 
-    if (grp.back_proto_len > 0) {
-      for (DrawPrototype &proto : prototypes.slice_safe({offset, grp.back_proto_len})) {
+    if (grp.back_facing_counter > 0) {
+      for (DrawPrototype &proto : prototypes.slice_safe({offset, grp.back_facing_counter})) {
         BLI_assert(proto.group_id == group_index);
-        ResourceHandle handle(proto.resource_handle);
+        ResourceHandle handle(proto.res_handle);
         BLI_assert(handle.has_inverted_handedness());
         ss << std::endl
            << line_prefix << "    .proto(instance_len=" << std::to_string(proto.instance_len)
            << ", resource_id=" << std::to_string(handle.resource_index()) << ", back_face)";
       }
-      offset += grp.back_proto_len;
+      offset += grp.back_facing_counter;
     }
 
-    if (grp.front_proto_len > 0) {
-      for (DrawPrototype &proto : prototypes.slice_safe({offset, grp.front_proto_len})) {
+    if (grp.front_facing_counter > 0) {
+      for (DrawPrototype &proto : prototypes.slice_safe({offset, grp.front_facing_counter})) {
         BLI_assert(proto.group_id == group_index);
-        ResourceHandle handle(proto.resource_handle);
+        ResourceHandle handle(proto.res_handle);
         BLI_assert(!handle.has_inverted_handedness());
         ss << std::endl
            << line_prefix << "    .proto(instance_len=" << std::to_string(proto.instance_len)
@@ -564,16 +736,11 @@ void DrawCommandBuf::finalize_commands(Vector<Header, 0> &headers,
       cmd.vertex_len = batch_vert_len;
     }
 
-#ifdef WITH_METAL_BACKEND
-    /* For SSBO vertex fetch, mutate output vertex count by ssbo vertex fetch expansion factor. */
-    if (cmd.shader) {
-      int num_input_primitives = gpu_get_prim_count_from_type(cmd.vertex_len,
-                                                              cmd.batch->prim_type);
-      cmd.vertex_len = num_input_primitives *
-                       GPU_shader_get_ssbo_vertex_fetch_num_verts_per_prim(cmd.shader);
-    }
-#endif
-
+    /* NOTE: Only do this if a handle is present. If a drawcall is using instancing with null
+     * handle, the shader should not rely on `resource_id` at ***all***. This allows procedural
+     * instanced drawcalls with lots of instances with no overhead. */
+    /* TODO(fclem): Think about either fixing this feature or removing support for instancing all
+     * together. */
     if (cmd.handle.raw > 0) {
       /* Save correct offset to start of resource_id buffer region for this draw. */
       uint instance_first = resource_id_count;
@@ -590,17 +757,20 @@ void DrawCommandBuf::finalize_commands(Vector<Header, 0> &headers,
   }
 }
 
-void DrawCommandBuf::bind(RecordingState &state,
-                          Vector<Header, 0> &headers,
-                          Vector<Undetermined, 0> &commands,
-                          SubPassVector &sub_passes)
+void DrawCommandBuf::generate_commands(Vector<Header, 0> &headers,
+                                       Vector<Undetermined, 0> &commands,
+                                       SubPassVector &sub_passes)
 {
-  resource_id_count_ = 0;
-
+  /* First instance ID contains the null handle with identity transform.
+   * This is referenced for drawcalls with no handle. */
+  resource_id_buf_.get_or_resize(0) = 0;
+  resource_id_count_ = 1;
   finalize_commands(headers, commands, sub_passes, resource_id_count_, resource_id_buf_);
-
   resource_id_buf_.push_update();
+}
 
+void DrawCommandBuf::bind(RecordingState &state)
+{
   if (GPU_shader_draw_parameters_support() == false) {
     state.resource_id_buf = resource_id_buf_;
   }
@@ -609,13 +779,12 @@ void DrawCommandBuf::bind(RecordingState &state,
   }
 }
 
-void DrawMultiBuf::bind(RecordingState &state,
-                        Vector<Header, 0> & /*headers*/,
-                        Vector<Undetermined, 0> & /*commands*/,
-                        VisibilityBuf &visibility_buf,
-                        int visibility_word_per_draw,
-                        int view_len,
-                        bool use_custom_ids)
+void DrawMultiBuf::generate_commands(Vector<Header, 0> & /*headers*/,
+                                     Vector<Undetermined, 0> & /*commands*/,
+                                     VisibilityBuf &visibility_buf,
+                                     int visibility_word_per_draw,
+                                     int view_len,
+                                     bool use_custom_ids)
 {
   GPU_debug_group_begin("DrawMultiBuf.bind");
 
@@ -623,44 +792,49 @@ void DrawMultiBuf::bind(RecordingState &state,
   for (DrawGroup &group : MutableSpan<DrawGroup>(group_buf_.data(), group_count_)) {
     /* Compute prefix sum of all instance of previous group. */
     group.start = resource_id_count_;
-    resource_id_count_ += group.len * view_len;
+    resource_id_count_ += group.len;
 
     int batch_vert_len, batch_vert_first, batch_base_index, batch_inst_len;
     /* Now that GPUBatches are guaranteed to be finished, extract their parameters. */
-    GPU_batch_draw_parameter_get(
-        group.gpu_batch, &batch_vert_len, &batch_vert_first, &batch_base_index, &batch_inst_len);
+    GPU_batch_draw_parameter_get(group.desc.gpu_batch,
+                                 &batch_vert_len,
+                                 &batch_vert_first,
+                                 &batch_base_index,
+                                 &batch_inst_len);
 
-    group.vertex_len = group.vertex_len == -1 ? batch_vert_len : group.vertex_len;
-    group.vertex_first = group.vertex_first == -1 ? batch_vert_first : group.vertex_first;
+    group.vertex_len = group.desc.vertex_len == 0 ? batch_vert_len : group.desc.vertex_len;
+    group.vertex_first = group.desc.vertex_first == -1 ? batch_vert_first :
+                                                         group.desc.vertex_first;
     group.base_index = batch_base_index;
-
-#ifdef WITH_METAL_BACKEND
-    /* For SSBO vertex fetch, mutate output vertex count by ssbo vertex fetch expansion factor. */
-    if (group.gpu_shader) {
-      int num_input_primitives = gpu_get_prim_count_from_type(group.vertex_len,
-                                                              group.gpu_batch->prim_type);
-      group.vertex_len = num_input_primitives *
-                         GPU_shader_get_ssbo_vertex_fetch_num_verts_per_prim(group.gpu_shader);
-      /* Override base index to -1, as all SSBO calls are submitted as non-indexed, with the
-       * index buffer indirection handled within the implemnetation. This is to ensure
-       * command generation can correctly assigns baseInstance in the non-indexed formatting. */
-      group.base_index = -1;
-    }
-#endif
-
     /* Instancing attributes are not supported using the new pipeline since we use the base
      * instance to set the correct resource_id. Workaround is a storage_buf + gl_InstanceID. */
     BLI_assert(batch_inst_len == 1);
     UNUSED_VARS_NDEBUG(batch_inst_len);
 
-    /* Now that we got the batch information, we can set the counters to 0. */
+    if (group.desc.expand_prim_type != GPU_PRIM_NONE) {
+      /* Expanded drawcall. */
+      IndexRange vert_range = GPU_batch_draw_expanded_parameter_get(
+          group.desc.gpu_batch->prim_type,
+          GPUPrimType(group.desc.expand_prim_type),
+          group.vertex_len,
+          group.vertex_first,
+          group.desc.expand_prim_len);
+
+      group.vertex_first = vert_range.start();
+      group.vertex_len = vert_range.size();
+      /* Override base index to -1 as the generated drawcall will not use an index buffer and do
+       * the indirection manually inside the shader. */
+      group.base_index = -1;
+    }
+
+    /* Reset counters to 0 for the GPU. */
     group.total_counter = group.front_facing_counter = group.back_facing_counter = 0;
   }
 
   group_buf_.push_update();
   prototype_buf_.push_update();
   /* Allocate enough for the expansion pass. */
-  resource_id_buf_.get_or_resize(resource_id_count_ * (use_custom_ids ? 2 : 1));
+  resource_id_buf_.get_or_resize(resource_id_count_ * view_len * (use_custom_ids ? 2 : 1));
   /* Two commands per group (inverted and non-inverted scale). */
   command_buf_.get_or_resize(group_count_ * 2);
 
@@ -669,6 +843,7 @@ void DrawMultiBuf::bind(RecordingState &state,
     GPU_shader_bind(shader);
     GPU_shader_uniform_1i(shader, "prototype_len", prototype_count_);
     GPU_shader_uniform_1i(shader, "visibility_word_per_draw", visibility_word_per_draw);
+    GPU_shader_uniform_1i(shader, "view_len", view_len);
     GPU_shader_uniform_1i(shader, "view_shift", log2_ceil_u(view_len));
     GPU_shader_uniform_1b(shader, "use_custom_ids", use_custom_ids);
     GPU_storagebuf_bind(group_buf_, GPU_shader_get_ssbo_binding(shader, "group_buf"));
@@ -677,18 +852,29 @@ void DrawMultiBuf::bind(RecordingState &state,
     GPU_storagebuf_bind(command_buf_, GPU_shader_get_ssbo_binding(shader, "command_buf"));
     GPU_storagebuf_bind(resource_id_buf_, DRW_RESOURCE_ID_SLOT);
     GPU_compute_dispatch(shader, divide_ceil_u(prototype_count_, DRW_COMMAND_GROUP_SIZE), 1, 1);
+    /* TODO(@fclem): Investigate moving the barrier in the bind function. */
     if (GPU_shader_draw_parameters_support() == false) {
       GPU_memory_barrier(GPU_BARRIER_VERTEX_ATTRIB_ARRAY);
-      state.resource_id_buf = resource_id_buf_;
     }
     else {
       GPU_memory_barrier(GPU_BARRIER_SHADER_STORAGE);
     }
+    GPU_storagebuf_sync_as_indirect_buffer(command_buf_);
   }
 
   GPU_debug_group_end();
 }
 
+void DrawMultiBuf::bind(RecordingState &state)
+{
+  if (GPU_shader_draw_parameters_support() == false) {
+    state.resource_id_buf = resource_id_buf_;
+  }
+  else {
+    GPU_storagebuf_bind(resource_id_buf_, DRW_RESOURCE_ID_SLOT);
+  }
+}
+
 /** \} */
 
 };  // namespace blender::draw::command
