@@ -6,43 +6,41 @@
 #include "BLI_virtual_array.hh"
 
 #include "BKE_attribute.hh"
-#include "BKE_compute_contexts.hh"
-#include "BKE_context.h"
+#include "BKE_context.hh"
 #include "BKE_curves.hh"
-#include "BKE_editmesh.h"
+#include "BKE_editmesh.hh"
 #include "BKE_geometry_fields.hh"
-#include "BKE_global.h"
+#include "BKE_geometry_set.hh"
+#include "BKE_geometry_set_instances.hh"
+#include "BKE_global.hh"
+#include "BKE_grease_pencil.hh"
 #include "BKE_instances.hh"
-#include "BKE_lib_id.h"
+#include "BKE_lib_id.hh"
 #include "BKE_mesh.hh"
 #include "BKE_mesh_wrapper.hh"
-#include "BKE_modifier.h"
-#include "BKE_volume.h"
-#include "BKE_volume_openvdb.hh"
+#include "BKE_modifier.hh"
+#include "BKE_object_types.hh"
+#include "BKE_volume.hh"
+#include "BKE_volume_grid.hh"
 
-#include "DNA_ID.h"
-#include "DNA_mesh_types.h"
-#include "DNA_meshdata_types.h"
 #include "DNA_pointcloud_types.h"
 #include "DNA_space_types.h"
-#include "DNA_userdef_types.h"
 
 #include "DEG_depsgraph_query.hh"
 
 #include "ED_curves.hh"
-#include "ED_spreadsheet.hh"
+#include "ED_outliner.hh"
 
-#include "NOD_geometry_nodes_lazy_function.hh"
 #include "NOD_geometry_nodes_log.hh"
 
-#include "BLT_translation.h"
+#include "BLT_translation.hh"
 
 #include "RNA_access.hh"
 #include "RNA_enum_types.hh"
 
-#include "FN_field_cpp_type.hh"
+#include "UI_resources.hh"
 
-#include "bmesh.h"
+#include "bmesh.hh"
 
 #include "spreadsheet_data_source_geometry.hh"
 #include "spreadsheet_intern.hh"
@@ -73,29 +71,29 @@ std::unique_ptr<ColumnValues> ExtraColumns::get_column_values(
 
 static void add_mesh_debug_column_names(
     const Mesh &mesh,
-    const eAttrDomain domain,
+    const bke::AttrDomain domain,
     FunctionRef<void(const SpreadsheetColumnID &, bool is_extra)> fn)
 {
   switch (domain) {
-    case ATTR_DOMAIN_POINT:
+    case bke::AttrDomain::Point:
       if (CustomData_has_layer(&mesh.vert_data, CD_ORIGINDEX)) {
         fn({(char *)"Original Index"}, false);
       }
       break;
-    case ATTR_DOMAIN_EDGE:
+    case bke::AttrDomain::Edge:
       if (CustomData_has_layer(&mesh.edge_data, CD_ORIGINDEX)) {
         fn({(char *)"Original Index"}, false);
       }
       fn({(char *)"Vertices"}, false);
       break;
-    case ATTR_DOMAIN_FACE:
+    case bke::AttrDomain::Face:
       if (CustomData_has_layer(&mesh.face_data, CD_ORIGINDEX)) {
         fn({(char *)"Original Index"}, false);
       }
       fn({(char *)"Corner Start"}, false);
       fn({(char *)"Corner Size"}, false);
       break;
-    case ATTR_DOMAIN_CORNER:
+    case bke::AttrDomain::Corner:
       fn({(char *)"Vertex"}, false);
       fn({(char *)"Edge"}, false);
       break;
@@ -106,26 +104,28 @@ static void add_mesh_debug_column_names(
 }
 
 static std::unique_ptr<ColumnValues> build_mesh_debug_columns(const Mesh &mesh,
-                                                              const eAttrDomain domain,
+                                                              const bke::AttrDomain domain,
                                                               const StringRef name)
 {
   switch (domain) {
-    case ATTR_DOMAIN_POINT: {
+    case bke::AttrDomain::Point: {
       if (name == "Original Index") {
         const int *data = static_cast<const int *>(
             CustomData_get_layer(&mesh.vert_data, CD_ORIGINDEX));
         if (data) {
-          return std::make_unique<ColumnValues>(name, VArray<int>::ForSpan({data, mesh.totvert}));
+          return std::make_unique<ColumnValues>(name,
+                                                VArray<int>::ForSpan({data, mesh.verts_num}));
         }
       }
       return {};
     }
-    case ATTR_DOMAIN_EDGE: {
+    case bke::AttrDomain::Edge: {
       if (name == "Original Index") {
         const int *data = static_cast<const int *>(
             CustomData_get_layer(&mesh.edge_data, CD_ORIGINDEX));
         if (data) {
-          return std::make_unique<ColumnValues>(name, VArray<int>::ForSpan({data, mesh.totedge}));
+          return std::make_unique<ColumnValues>(name,
+                                                VArray<int>::ForSpan({data, mesh.edges_num}));
         }
       }
       if (name == "Vertices") {
@@ -133,7 +133,7 @@ static std::unique_ptr<ColumnValues> build_mesh_debug_columns(const Mesh &mesh,
       }
       return {};
     }
-    case ATTR_DOMAIN_FACE: {
+    case bke::AttrDomain::Face: {
       if (name == "Original Index") {
         const int *data = static_cast<const int *>(
             CustomData_get_layer(&mesh.face_data, CD_ORIGINDEX));
@@ -155,7 +155,7 @@ static std::unique_ptr<ColumnValues> build_mesh_debug_columns(const Mesh &mesh,
       }
       return {};
     }
-    case ATTR_DOMAIN_CORNER: {
+    case bke::AttrDomain::Corner: {
       if (name == "Vertex") {
         return std::make_unique<ColumnValues>(name, VArray<int>::ForSpan(mesh.corner_verts()));
       }
@@ -173,12 +173,11 @@ static std::unique_ptr<ColumnValues> build_mesh_debug_columns(const Mesh &mesh,
 void GeometryDataSource::foreach_default_column_ids(
     FunctionRef<void(const SpreadsheetColumnID &, bool is_extra)> fn) const
 {
-  if (!component_->attributes().has_value()) {
+  std::optional<const bke::AttributeAccessor> attributes = this->get_component_attributes();
+  if (!attributes.has_value()) {
     return;
   }
-  const bke::AttributeAccessor attributes = *component_->attributes();
-
-  if (attributes.domain_size(domain_) == 0) {
+  if (attributes->domain_size(domain_) == 0) {
     return;
   }
 
@@ -186,27 +185,35 @@ void GeometryDataSource::foreach_default_column_ids(
     fn({(char *)"Name"}, false);
   }
 
+  if (component_->type() == bke::GeometryComponent::Type::GreasePencil) {
+    fn({(char *)"Name"}, false);
+  }
+
   extra_columns_.foreach_default_column_ids(fn);
 
-  attributes.for_all(
-      [&](const bke::AttributeIDRef &attribute_id, const bke::AttributeMetaData &meta_data) {
-        if (meta_data.domain != domain_) {
-          return true;
-        }
-        if (attribute_id.is_anonymous()) {
-          return true;
-        }
-        if (!bke::allow_procedural_attribute_access(attribute_id.name())) {
-          return true;
-        }
-        SpreadsheetColumnID column_id;
-        column_id.name = (char *)attribute_id.name().data();
-        const bool is_front = attribute_id.name() == ".viewer";
-        fn(column_id, is_front);
-        return true;
-      });
+  attributes->foreach_attribute([&](const bke::AttributeIter &iter) {
+    if (iter.domain != domain_) {
+      return;
+    }
+    if (bke::attribute_name_is_anonymous(iter.name)) {
+      return;
+    }
+    if (!bke::allow_procedural_attribute_access(iter.name)) {
+      return;
+    }
+    if (iter.domain == bke::AttrDomain::Instance && iter.name == "instance_transform") {
+      /* Don't display the instance transform attribute, since matrix visualization in the
+       * spreadsheet isn't helpful. */
+      return;
+    }
+    SpreadsheetColumnID column_id;
+    column_id.name = (char *)iter.name.data();
+    const bool is_front = iter.name == ".viewer";
+    fn(column_id, is_front);
+  });
 
   if (component_->type() == bke::GeometryComponent::Type::Instance) {
+    fn({(char *)"Position"}, false);
     fn({(char *)"Rotation"}, false);
     fn({(char *)"Scale"}, false);
   }
@@ -221,11 +228,11 @@ void GeometryDataSource::foreach_default_column_ids(
 std::unique_ptr<ColumnValues> GeometryDataSource::get_column_values(
     const SpreadsheetColumnID &column_id) const
 {
-  if (!component_->attributes().has_value()) {
+  std::optional<const bke::AttributeAccessor> attributes = this->get_component_attributes();
+  if (!attributes.has_value()) {
     return {};
   }
-  const bke::AttributeAccessor attributes = *component_->attributes();
-  const int domain_num = attributes.domain_size(domain_);
+  const int domain_num = attributes->domain_size(domain_);
   if (domain_num == 0) {
     return {};
   }
@@ -252,6 +259,12 @@ std::unique_ptr<ColumnValues> GeometryDataSource::get_column_values(
                 }));
       }
       Span<float4x4> transforms = instances->transforms();
+      if (STREQ(column_id.name, "Position")) {
+        return std::make_unique<ColumnValues>(
+            column_id.name, VArray<float3>::ForFunc(domain_num, [transforms](int64_t index) {
+              return transforms[index].location();
+            }));
+      }
       if (STREQ(column_id.name, "Rotation")) {
         return std::make_unique<ColumnValues>(
             column_id.name, VArray<float3>::ForFunc(domain_num, [transforms](int64_t index) {
@@ -261,7 +274,24 @@ std::unique_ptr<ColumnValues> GeometryDataSource::get_column_values(
       if (STREQ(column_id.name, "Scale")) {
         return std::make_unique<ColumnValues>(
             column_id.name, VArray<float3>::ForFunc(domain_num, [transforms](int64_t index) {
-              return math::to_scale(transforms[index]);
+              return math::to_scale<true>(transforms[index]);
+            }));
+      }
+    }
+  }
+  else if (component_->type() == bke::GeometryComponent::Type::GreasePencil) {
+    if (const GreasePencil *grease_pencil =
+            static_cast<const bke::GreasePencilComponent &>(*component_).get())
+    {
+      if (domain_ == bke::AttrDomain::Layer && STREQ(column_id.name, "Name")) {
+        const Span<const bke::greasepencil::Layer *> layers = grease_pencil->layers();
+        return std::make_unique<ColumnValues>(
+            column_id.name, VArray<std::string>::ForFunc(domain_num, [layers](int64_t index) {
+              StringRefNull name = layers[index]->name();
+              if (name.is_empty()) {
+                name = IFACE_("(Layer)");
+              }
+              return std::string(name);
             }));
       }
     }
@@ -277,7 +307,7 @@ std::unique_ptr<ColumnValues> GeometryDataSource::get_column_values(
     }
   }
 
-  bke::GAttributeReader attribute = attributes.lookup(column_id.name);
+  bke::GAttributeReader attribute = attributes->lookup(column_id.name);
   if (!attribute) {
     return {};
   }
@@ -296,40 +326,42 @@ std::unique_ptr<ColumnValues> GeometryDataSource::get_column_values(
 
 int GeometryDataSource::tot_rows() const
 {
-  if (!component_->attributes().has_value()) {
+  std::optional<const bke::AttributeAccessor> attributes = this->get_component_attributes();
+  if (!attributes.has_value()) {
     return {};
   }
-  const bke::AttributeAccessor attributes = *component_->attributes();
-  return attributes.domain_size(domain_);
+  return attributes->domain_size(domain_);
 }
 
 bool GeometryDataSource::has_selection_filter() const
 {
-  Object *object_orig = DEG_get_original_object(object_eval_);
+  if (!object_orig_) {
+    return false;
+  }
   switch (component_->type()) {
     case bke::GeometryComponent::Type::Mesh: {
-      if (object_orig->type != OB_MESH) {
+      if (object_orig_->type != OB_MESH) {
         return false;
       }
-      if (object_orig->mode != OB_MODE_EDIT) {
+      if (object_orig_->mode != OB_MODE_EDIT) {
         return false;
       }
       return true;
     }
     case bke::GeometryComponent::Type::Curve: {
-      if (object_orig->type != OB_CURVES) {
+      if (object_orig_->type != OB_CURVES) {
         return false;
       }
-      if (!ELEM(object_orig->mode, OB_MODE_SCULPT_CURVES, OB_MODE_EDIT)) {
+      if (!ELEM(object_orig_->mode, OB_MODE_SCULPT_CURVES, OB_MODE_EDIT)) {
         return false;
       }
       return true;
     }
     case bke::GeometryComponent::Type::PointCloud: {
-      if (object_orig->type != OB_POINTCLOUD) {
+      if (object_orig_->type != OB_POINTCLOUD) {
         return false;
       }
-      if (object_orig->mode != OB_MODE_EDIT) {
+      if (object_orig_->mode != OB_MODE_EDIT) {
         return false;
       }
       return true;
@@ -339,6 +371,117 @@ bool GeometryDataSource::has_selection_filter() const
   }
 }
 
+static IndexMask calc_mesh_selection_mask_faces(const Mesh &mesh_eval,
+                                                const Mesh &mesh_orig,
+                                                IndexMaskMemory &memory)
+{
+  const bke::AttributeAccessor attributes_eval = mesh_eval.attributes();
+  const IndexRange range(attributes_eval.domain_size(bke::AttrDomain::Face));
+  BMesh *bm = mesh_orig.runtime->edit_mesh->bm;
+
+  BM_mesh_elem_table_ensure(bm, BM_FACE);
+  if (mesh_eval.faces_num == bm->totface) {
+    return IndexMask::from_predicate(range, GrainSize(4096), memory, [&](const int i) {
+      const BMFace *face = BM_face_at_index(bm, i);
+      return BM_elem_flag_test_bool(face, BM_ELEM_SELECT);
+    });
+  }
+  if (const int *orig_indices = static_cast<const int *>(
+          CustomData_get_layer(&mesh_eval.face_data, CD_ORIGINDEX)))
+  {
+    return IndexMask::from_predicate(range, GrainSize(2048), memory, [&](const int i) {
+      const int orig = orig_indices[i];
+      if (orig == -1) {
+        return false;
+      }
+      const BMFace *face = BM_face_at_index(bm, orig);
+      return BM_elem_flag_test_bool(face, BM_ELEM_SELECT);
+    });
+  }
+  return range;
+}
+
+static IndexMask calc_mesh_selection_mask(const Mesh &mesh_eval,
+                                          const Mesh &mesh_orig,
+                                          const bke::AttrDomain domain,
+                                          IndexMaskMemory &memory)
+{
+  const bke::AttributeAccessor attributes_eval = mesh_eval.attributes();
+  const IndexRange range(attributes_eval.domain_size(domain));
+  BMesh *bm = mesh_orig.runtime->edit_mesh->bm;
+
+  switch (domain) {
+    case bke::AttrDomain::Point: {
+      BM_mesh_elem_table_ensure(bm, BM_VERT);
+      if (mesh_eval.verts_num == bm->totvert) {
+        return IndexMask::from_predicate(range, GrainSize(4096), memory, [&](const int i) {
+          const BMVert *vert = BM_vert_at_index(bm, i);
+          return BM_elem_flag_test_bool(vert, BM_ELEM_SELECT);
+        });
+      }
+      if (const int *orig_indices = static_cast<const int *>(
+              CustomData_get_layer(&mesh_eval.vert_data, CD_ORIGINDEX)))
+      {
+        return IndexMask::from_predicate(range, GrainSize(2048), memory, [&](const int i) {
+          const int orig = orig_indices[i];
+          if (orig == -1) {
+            return false;
+          }
+          const BMVert *vert = BM_vert_at_index(bm, orig);
+          return BM_elem_flag_test_bool(vert, BM_ELEM_SELECT);
+        });
+      }
+      return range;
+    }
+    case bke::AttrDomain::Edge: {
+      BM_mesh_elem_table_ensure(bm, BM_EDGE);
+      if (mesh_eval.edges_num == bm->totedge) {
+        return IndexMask::from_predicate(range, GrainSize(4096), memory, [&](const int i) {
+          const BMEdge *edge = BM_edge_at_index(bm, i);
+          return BM_elem_flag_test_bool(edge, BM_ELEM_SELECT);
+        });
+      }
+      if (const int *orig_indices = static_cast<const int *>(
+              CustomData_get_layer(&mesh_eval.edge_data, CD_ORIGINDEX)))
+      {
+        return IndexMask::from_predicate(range, GrainSize(2048), memory, [&](const int i) {
+          const int orig = orig_indices[i];
+          if (orig == -1) {
+            return false;
+          }
+          const BMEdge *edge = BM_edge_at_index(bm, orig);
+          return BM_elem_flag_test_bool(edge, BM_ELEM_SELECT);
+        });
+      }
+      return range;
+    }
+    case bke::AttrDomain::Face: {
+      return calc_mesh_selection_mask_faces(mesh_eval, mesh_orig, memory);
+    }
+    case bke::AttrDomain::Corner: {
+      IndexMaskMemory face_memory;
+      const IndexMask face_mask = calc_mesh_selection_mask_faces(
+          mesh_eval, mesh_orig, face_memory);
+      if (face_mask.is_empty()) {
+        return {};
+      }
+      if (face_mask.size() == range.size()) {
+        return range;
+      }
+
+      Array<bool> face_selection(range.size(), false);
+      face_mask.to_bools(face_selection);
+
+      const VArray<bool> corner_selection = attributes_eval.adapt_domain<bool>(
+          VArray<bool>::ForSpan(face_selection), bke::AttrDomain::Face, bke::AttrDomain::Corner);
+      return IndexMask::from_bools(corner_selection, memory);
+    }
+    default:
+      BLI_assert_unreachable();
+      return range;
+  }
+}
+
 IndexMask GeometryDataSource::apply_selection_filter(IndexMaskMemory &memory) const
 {
   std::lock_guard lock{mutex_};
@@ -349,60 +492,20 @@ IndexMask GeometryDataSource::apply_selection_filter(IndexMaskMemory &memory) co
 
   switch (component_->type()) {
     case bke::GeometryComponent::Type::Mesh: {
-      BLI_assert(object_eval_->type == OB_MESH);
-      BLI_assert(object_eval_->mode == OB_MODE_EDIT);
-      Object *object_orig = DEG_get_original_object(object_eval_);
+      BLI_assert(object_orig_->type == OB_MESH);
+      BLI_assert(object_orig_->mode == OB_MODE_EDIT);
       const Mesh *mesh_eval = geometry_set_.get_mesh();
-      const bke::AttributeAccessor attributes_eval = mesh_eval->attributes();
-      Mesh *mesh_orig = (Mesh *)object_orig->data;
-      BMesh *bm = mesh_orig->edit_mesh->bm;
-      BM_mesh_elem_table_ensure(bm, BM_VERT);
-
-      const int *orig_indices = (const int *)CustomData_get_layer(&mesh_eval->vert_data,
-                                                                  CD_ORIGINDEX);
-      if (orig_indices != nullptr) {
-        /* Use CD_ORIGINDEX layer if it exists. */
-        VArray<bool> selection = attributes_eval.adapt_domain<bool>(
-            VArray<bool>::ForFunc(mesh_eval->totvert,
-                                  [bm, orig_indices](int vertex_index) -> bool {
-                                    const int i_orig = orig_indices[vertex_index];
-                                    if (i_orig < 0) {
-                                      return false;
-                                    }
-                                    if (i_orig >= bm->totvert) {
-                                      return false;
-                                    }
-                                    const BMVert *vert = BM_vert_at_index(bm, i_orig);
-                                    return BM_elem_flag_test(vert, BM_ELEM_SELECT);
-                                  }),
-            ATTR_DOMAIN_POINT,
-            domain_);
-        return IndexMask::from_bools(selection, memory);
-      }
-
-      if (mesh_eval->totvert == bm->totvert) {
-        /* Use a simple heuristic to match original vertices to evaluated ones. */
-        VArray<bool> selection = attributes_eval.adapt_domain<bool>(
-            VArray<bool>::ForFunc(mesh_eval->totvert,
-                                  [bm](int vertex_index) -> bool {
-                                    const BMVert *vert = BM_vert_at_index(bm, vertex_index);
-                                    return BM_elem_flag_test(vert, BM_ELEM_SELECT);
-                                  }),
-            ATTR_DOMAIN_POINT,
-            domain_);
-        return IndexMask::from_bools(selection, memory);
-      }
-
-      return full_range;
+      const Mesh *mesh_orig = static_cast<const Mesh *>(object_orig_->data);
+      return calc_mesh_selection_mask(*mesh_eval, *mesh_orig, domain_, memory);
     }
     case bke::GeometryComponent::Type::Curve: {
-      BLI_assert(object_eval_->type == OB_CURVES);
+      BLI_assert(object_orig_->type == OB_CURVES);
       const bke::CurveComponent &component = static_cast<const bke::CurveComponent &>(*component_);
       const Curves &curves_id = *component.get();
       switch (domain_) {
-        case ATTR_DOMAIN_POINT:
+        case bke::AttrDomain::Point:
           return curves::retrieve_selected_points(curves_id, memory);
-        case ATTR_DOMAIN_CURVE:
+        case bke::AttrDomain::Curve:
           return curves::retrieve_selected_curves(curves_id, memory);
         default:
           BLI_assert_unreachable();
@@ -410,10 +513,10 @@ IndexMask GeometryDataSource::apply_selection_filter(IndexMaskMemory &memory) co
       return full_range;
     }
     case bke::GeometryComponent::Type::PointCloud: {
-      BLI_assert(object_eval_->type == OB_POINTCLOUD);
+      BLI_assert(object_orig_->type == OB_POINTCLOUD);
       const bke::AttributeAccessor attributes = *component_->attributes();
-      const VArray<bool> &selection = *attributes.lookup_or_default(
-          ".selection", ATTR_DOMAIN_POINT, false);
+      const VArray<bool> selection = *attributes.lookup_or_default(
+          ".selection", bke::AttrDomain::Point, true);
       return IndexMask::from_bools(selection, memory);
     }
     default:
@@ -421,6 +524,28 @@ IndexMask GeometryDataSource::apply_selection_filter(IndexMaskMemory &memory) co
   }
 }
 
+std::optional<const bke::AttributeAccessor> GeometryDataSource::get_component_attributes() const
+{
+  if (component_->type() != bke::GeometryComponent::Type::GreasePencil) {
+    return component_->attributes();
+  }
+  const GreasePencil *grease_pencil = geometry_set_.get_grease_pencil();
+  if (!grease_pencil) {
+    return {};
+  }
+  if (domain_ == bke::AttrDomain::Layer) {
+    return grease_pencil->attributes();
+  }
+  if (layer_index_ >= 0 && layer_index_ < grease_pencil->layers().size()) {
+    if (const bke::greasepencil::Drawing *drawing = grease_pencil->get_eval_drawing(
+            grease_pencil->layer(layer_index_)))
+    {
+      return drawing->strokes().attributes();
+    }
+  }
+  return {};
+}
+
 void VolumeDataSource::foreach_default_column_ids(
     FunctionRef<void(const SpreadsheetColumnID &, bool is_extra)> fn) const
 {
@@ -447,15 +572,15 @@ std::unique_ptr<ColumnValues> VolumeDataSource::get_column_values(
   if (STREQ(column_id.name, "Grid Name")) {
     return std::make_unique<ColumnValues>(
         IFACE_("Grid Name"), VArray<std::string>::ForFunc(size, [volume](int64_t index) {
-          const VolumeGrid *volume_grid = BKE_volume_grid_get_for_read(volume, index);
-          return BKE_volume_grid_name(volume_grid);
+          const bke::VolumeGridData *volume_grid = BKE_volume_grid_get(volume, index);
+          return volume_grid->name();
         }));
   }
   if (STREQ(column_id.name, "Data Type")) {
     return std::make_unique<ColumnValues>(
         IFACE_("Data Type"), VArray<std::string>::ForFunc(size, [volume](int64_t index) {
-          const VolumeGrid *volume_grid = BKE_volume_grid_get_for_read(volume, index);
-          const VolumeGridType type = BKE_volume_grid_type(volume_grid);
+          const bke::VolumeGridData *volume_grid = BKE_volume_grid_get(volume, index);
+          const VolumeGridType type = volume_grid->grid_type();
           const char *name = nullptr;
           RNA_enum_name_from_value(rna_enum_volume_grid_data_type_items, type, &name);
           return IFACE_(name);
@@ -464,9 +589,8 @@ std::unique_ptr<ColumnValues> VolumeDataSource::get_column_values(
   if (STREQ(column_id.name, "Class")) {
     return std::make_unique<ColumnValues>(
         IFACE_("Class"), VArray<std::string>::ForFunc(size, [volume](int64_t index) {
-          const VolumeGrid *volume_grid = BKE_volume_grid_get_for_read(volume, index);
-          openvdb::GridBase::ConstPtr grid = BKE_volume_grid_openvdb_for_read(volume, volume_grid);
-          openvdb::GridClass grid_class = grid->getGridClass();
+          const bke::VolumeGridData *volume_grid = BKE_volume_grid_get(volume, index);
+          openvdb::GridClass grid_class = volume_grid->grid_class();
           if (grid_class == openvdb::GridClass::GRID_FOG_VOLUME) {
             return IFACE_("Fog Volume");
           }
@@ -492,6 +616,26 @@ int VolumeDataSource::tot_rows() const
   return BKE_volume_num_grids(volume);
 }
 
+int get_instance_reference_icon(const bke::InstanceReference &reference)
+{
+  switch (reference.type()) {
+    case bke::InstanceReference::Type::Object: {
+      const Object &object = reference.object();
+      return ED_outliner_icon_from_id(object.id);
+    }
+    case bke::InstanceReference::Type::Collection: {
+      return ICON_OUTLINER_COLLECTION;
+    }
+    case bke::InstanceReference::Type::GeometrySet: {
+      return ICON_GEOMETRY_SET;
+    }
+    case bke::InstanceReference::Type::None: {
+      break;
+    }
+  }
+  return ICON_NONE;
+}
+
 bke::GeometrySet spreadsheet_get_display_geometry_set(const SpaceSpreadsheet *sspreadsheet,
                                                       Object *object_eval)
 {
@@ -501,12 +645,12 @@ bke::GeometrySet spreadsheet_get_display_geometry_set(const SpaceSpreadsheet *ss
     if (object_orig->type == OB_MESH) {
       const Mesh *mesh = static_cast<const Mesh *>(object_orig->data);
       if (object_orig->mode == OB_MODE_EDIT) {
-        if (const BMEditMesh *em = mesh->edit_mesh) {
+        if (const BMEditMesh *em = mesh->runtime->edit_mesh.get()) {
           Mesh *new_mesh = (Mesh *)BKE_id_new_nomain(ID_ME, nullptr);
           /* This is a potentially heavy operation to do on every redraw. The best solution here is
            * to display the data directly from the bmesh without a conversion, which can be
            * implemented a bit later. */
-          BM_mesh_bm_to_me_for_eval(em->bm, new_mesh, nullptr);
+          BM_mesh_bm_to_me_for_eval(*em->bm, *new_mesh, nullptr);
           geometry_set.replace_mesh(new_mesh, bke::GeometryOwnershipType::Owned);
         }
       }
@@ -524,19 +668,15 @@ bke::GeometrySet spreadsheet_get_display_geometry_set(const SpaceSpreadsheet *ss
       geometry_set.replace_curves(&const_cast<Curves &>(curves_id),
                                   bke::GeometryOwnershipType::ReadOnly);
     }
+    else if (object_orig->type == OB_GREASE_PENCIL) {
+      const GreasePencil &grease_pencil = *static_cast<const GreasePencil *>(object_orig->data);
+      geometry_set.replace_grease_pencil(&const_cast<GreasePencil &>(grease_pencil),
+                                         bke::GeometryOwnershipType::ReadOnly);
+    }
   }
   else {
     if (BLI_listbase_is_single(&sspreadsheet->viewer_path.path)) {
-      if (const bke::GeometrySet *geometry_eval = object_eval->runtime.geometry_set_eval) {
-        geometry_set = *geometry_eval;
-      }
-
-      if (object_eval->mode == OB_MODE_EDIT && object_eval->type == OB_MESH) {
-        if (Mesh *mesh = BKE_modifier_get_evaluated_mesh_from_evaluated_object(object_eval)) {
-          BKE_mesh_wrapper_ensure_mdata(mesh);
-          geometry_set.replace_mesh(mesh, bke::GeometryOwnershipType::ReadOnly);
-        }
-      }
+      geometry_set = bke::object_get_evaluated_geometry_set(*object_eval);
     }
     else {
       if (const ViewerNodeLog *viewer_log =
@@ -550,12 +690,41 @@ bke::GeometrySet spreadsheet_get_display_geometry_set(const SpaceSpreadsheet *ss
   return geometry_set;
 }
 
+bke::GeometrySet get_geometry_set_for_instance_ids(const bke::GeometrySet &root_geometry,
+                                                   const Span<SpreadsheetInstanceID> instance_ids)
+{
+  bke::GeometrySet geometry = root_geometry;
+  for (const SpreadsheetInstanceID &instance_id : instance_ids) {
+    const bke::Instances *instances = geometry.get_instances();
+    if (!instances) {
+      /* Return the best available geometry. */
+      return geometry;
+    }
+    const Span<bke::InstanceReference> references = instances->references();
+    if (instance_id.reference_index < 0 || instance_id.reference_index >= references.size()) {
+      /* Return the best available geometry. */
+      return geometry;
+    }
+    const bke::InstanceReference &reference = references[instance_id.reference_index];
+    bke::GeometrySet reference_geometry;
+    reference.to_geometry_set(reference_geometry);
+    geometry = reference_geometry;
+  }
+  return geometry;
+}
+
 std::unique_ptr<DataSource> data_source_from_geometry(const bContext *C, Object *object_eval)
 {
   SpaceSpreadsheet *sspreadsheet = CTX_wm_space_spreadsheet(C);
-  const eAttrDomain domain = (eAttrDomain)sspreadsheet->attribute_domain;
+
+  const bke::GeometrySet root_geometry_set = spreadsheet_get_display_geometry_set(sspreadsheet,
+                                                                                  object_eval);
+  const bke::GeometrySet geometry_set = get_geometry_set_for_instance_ids(
+      root_geometry_set, Span{sspreadsheet->instance_ids, sspreadsheet->instance_ids_num});
+
+  const bke::AttrDomain domain = (bke::AttrDomain)sspreadsheet->attribute_domain;
   const auto component_type = bke::GeometryComponent::Type(sspreadsheet->geometry_component_type);
-  bke::GeometrySet geometry_set = spreadsheet_get_display_geometry_set(sspreadsheet, object_eval);
+  const int active_layer_index = sspreadsheet->active_layer_index;
   if (!geometry_set.has(component_type)) {
     return {};
   }
@@ -563,8 +732,11 @@ std::unique_ptr<DataSource> data_source_from_geometry(const bContext *C, Object
   if (component_type == bke::GeometryComponent::Type::Volume) {
     return std::make_unique<VolumeDataSource>(std::move(geometry_set));
   }
+  Object *object_orig = sspreadsheet->instance_ids_num == 0 ?
+                            DEG_get_original_object(object_eval) :
+                            nullptr;
   return std::make_unique<GeometryDataSource>(
-      object_eval, std::move(geometry_set), component_type, domain);
+      object_orig, std::move(geometry_set), component_type, domain, active_layer_index);
 }
 
 }  // namespace blender::ed::spreadsheet
