@@ -18,26 +18,24 @@
 #include "BLI_math_matrix.h"
 #include "BLI_math_vector.h"
 
-#include "BKE_action.h"
-#include "BKE_anim_data.h"
-#include "BKE_context.h"
-#include "BKE_fcurve.h"
-#include "BKE_global.h"
-#include "BKE_image.h"
-#include "BKE_layer.h"
-#include "BKE_lib_id.h"
-#include "BKE_main.h"
-#include "BKE_modifier.h"
-#include "BKE_nla.h"
-#include "BKE_scene.h"
+#include "BKE_action.hh"
+#include "BKE_anim_data.hh"
+#include "BKE_context.hh"
+#include "BKE_global.hh"
+#include "BKE_layer.hh"
+#include "BKE_lib_id.hh"
+#include "BKE_modifier.hh"
+#include "BKE_nla.hh"
+#include "BKE_scene.hh"
 
-#include "ED_keyframes_edit.hh"
-#include "ED_keyframing.hh"
 #include "ED_particle.hh"
 #include "ED_screen.hh"
 #include "ED_screen_types.hh"
 #include "ED_sequencer.hh"
 
+#include "ANIM_keyframing.hh"
+#include "ANIM_nla.hh"
+
 #include "UI_view2d.hh"
 
 #include "WM_types.hh"
@@ -45,11 +43,12 @@
 #include "DEG_depsgraph_build.hh"
 
 #include "transform.hh"
-#include "transform_snap.hh"
 
 /* Own include. */
 #include "transform_convert.hh"
 
+using namespace blender;
+
 bool transform_mode_use_local_origins(const TransInfo *t)
 {
   return ELEM(t->mode, TFM_ROTATION, TFM_RESIZE, TFM_TRACKBALL);
@@ -171,6 +170,39 @@ static void sort_trans_data_selected_first(TransInfo *t)
   }
 }
 
+static float3 prop_dist_loc_get(const TransDataContainer *tc,
+                                const TransData *td,
+                                const bool use_island,
+                                const float proj_vec[3])
+{
+  float3 r_vec;
+
+  if (use_island) {
+    if (tc->use_local_mat) {
+      mul_v3_m4v3(r_vec, tc->mat, td->iloc);
+    }
+    else {
+      mul_v3_m3v3(r_vec, td->mtx, td->iloc);
+    }
+  }
+  else {
+    if (tc->use_local_mat) {
+      mul_v3_m4v3(r_vec, tc->mat, td->center);
+    }
+    else {
+      mul_v3_m3v3(r_vec, td->mtx, td->center);
+    }
+  }
+
+  if (proj_vec) {
+    float vec_p[3];
+    project_v3_v3v3(vec_p, r_vec, proj_vec);
+    sub_v3_v3(r_vec, vec_p);
+  }
+
+  return r_vec;
+}
+
 /**
  * Distance calculated from not-selected vertex to nearest selected vertex.
  */
@@ -181,7 +213,7 @@ static void set_prop_dist(TransInfo *t, const bool with_dist)
   float _proj_vec[3];
   const float *proj_vec = nullptr;
 
-  /* support for face-islands */
+  /* Support for face-islands. */
   const bool use_island = transdata_check_local_islands(t, t->around);
 
   if (t->flag & T_PROP_PROJECTED) {
@@ -221,31 +253,9 @@ static void set_prop_dist(TransInfo *t, const bool with_dist)
     for (a = 0; a < tc->data_len; a++, td++) {
       if (td->flag & TD_SELECTED) {
         /* Initialize, it was malloced. */
-        float vec[3];
         td->rdist = 0.0f;
 
-        if (use_island) {
-          if (tc->use_local_mat) {
-            mul_v3_m4v3(vec, tc->mat, td->iloc);
-          }
-          else {
-            mul_v3_m3v3(vec, td->mtx, td->iloc);
-          }
-        }
-        else {
-          if (tc->use_local_mat) {
-            mul_v3_m4v3(vec, tc->mat, td->center);
-          }
-          else {
-            mul_v3_m3v3(vec, td->mtx, td->center);
-          }
-        }
-
-        if (proj_vec) {
-          float vec_p[3];
-          project_v3_v3v3(vec_p, vec, proj_vec);
-          sub_v3_v3(vec, vec_p);
-        }
+        const float3 vec = prop_dist_loc_get(tc, td, use_island, proj_vec);
 
         BLI_kdtree_3d_insert(td_tree, td_table_index, vec);
         td_table[td_table_index++] = td;
@@ -265,30 +275,7 @@ static void set_prop_dist(TransInfo *t, const bool with_dist)
     TransData *td = tc->data;
     for (a = 0; a < tc->data_len; a++, td++) {
       if ((td->flag & TD_SELECTED) == 0) {
-        float vec[3];
-
-        if (use_island) {
-          if (tc->use_local_mat) {
-            mul_v3_m4v3(vec, tc->mat, td->iloc);
-          }
-          else {
-            mul_v3_m3v3(vec, td->mtx, td->iloc);
-          }
-        }
-        else {
-          if (tc->use_local_mat) {
-            mul_v3_m4v3(vec, tc->mat, td->center);
-          }
-          else {
-            mul_v3_m3v3(vec, td->mtx, td->center);
-          }
-        }
-
-        if (proj_vec) {
-          float vec_p[3];
-          project_v3_v3v3(vec_p, vec, proj_vec);
-          sub_v3_v3(vec, vec_p);
-        }
+        const float3 vec = prop_dist_loc_get(tc, td, use_island, proj_vec);
 
         KDTreeNearest_3d nearest;
         const int td_index = BLI_kdtree_3d_find_nearest(td_tree, vec, &nearest);
@@ -319,17 +306,17 @@ static void set_prop_dist(TransInfo *t, const bool with_dist)
 /** \name Pose Mode (Auto-IK)
  * \{ */
 
-/* adjust pose-channel's auto-ik chainlen */
+/** Adjust pose-channel's auto-ik chainlen. */
 static bool pchan_autoik_adjust(bPoseChannel *pchan, short chainlen)
 {
   bool changed = false;
 
-  /* don't bother to search if no valid constraints */
+  /* Don't bother to search if no valid constraints. */
   if ((pchan->constflag & (PCHAN_HAS_IK | PCHAN_HAS_NO_TARGET)) == 0) {
     return changed;
   }
 
-  /* check if pchan has ik-constraint */
+  /* Check if pchan has ik-constraint. */
   LISTBASE_FOREACH (bConstraint *, con, &pchan->constraints) {
     if (con->flag & (CONSTRAINT_DISABLE | CONSTRAINT_OFF)) {
       continue;
@@ -337,9 +324,9 @@ static bool pchan_autoik_adjust(bPoseChannel *pchan, short chainlen)
     if (con->type == CONSTRAINT_TYPE_KINEMATIC && (con->enforce != 0.0f)) {
       bKinematicConstraint *data = static_cast<bKinematicConstraint *>(con->data);
 
-      /* only accept if a temporary one (for auto-ik) */
+      /* Only accept if a temporary one (for auto-IK). */
       if (data->flag & CONSTRAINT_IK_TEMP) {
-        /* chainlen is new chainlen, but is limited by maximum chainlen */
+        /* `chainlen` is new `chainlen`, but is limited by maximum `chainlen`. */
         const int old_rootbone = data->rootbone;
         if ((chainlen == 0) || (chainlen > data->max_rootbone)) {
           data->rootbone = data->max_rootbone;
@@ -361,13 +348,13 @@ void transform_autoik_update(TransInfo *t, short mode)
 
   short *chainlen = &t->settings->autoik_chainlen;
 
-  /* mode determines what change to apply to chainlen */
+  /* `mode` determines what change to apply to `chainlen`. */
   if (mode == 1) {
-    /* mode=1 is from WHEELMOUSEDOWN... increases len */
+    /* `mode==1` is from WHEELMOUSEDOWN: increases len. */
     (*chainlen)++;
   }
   else if (mode == -1) {
-    /* mode==-1 is from WHEELMOUSEUP... decreases len */
+    /* `mode==-1` is from WHEELMOUSEUP: decreases len. */
     if (*chainlen > 0) {
       (*chainlen)--;
     }
@@ -377,12 +364,12 @@ void transform_autoik_update(TransInfo *t, short mode)
     }
   }
 
-  /* apply to all pose-channels */
+  /* Apply to all pose-channels. */
   bool changed = false;
 
   FOREACH_TRANS_DATA_CONTAINER (t, tc) {
 
-    /* sanity checks (don't assume t->poseobj is set, or that it is an armature) */
+    /* Sanity checks (don't assume `t->poseobj` is set, or that it is an armature). */
     if (ELEM(nullptr, tc->poseobj, tc->poseobj->pose)) {
       continue;
     }
@@ -474,7 +461,7 @@ TransDataCurveHandleFlags *initTransDataCurveHandles(TransData *td, BezTriple *b
       MEM_mallocN(sizeof(TransDataCurveHandleFlags), "CuHandle Data"));
   hdata->ih1 = bezt->h1;
   hdata->h1 = &bezt->h1;
-  hdata->ih2 = bezt->h2; /* in case the second is not selected */
+  hdata->ih2 = bezt->h2; /* In case the second is not selected. */
   hdata->h2 = &bezt->h2;
   return hdata;
 }
@@ -537,12 +524,12 @@ char transform_convert_frame_side_dir_get(TransInfo *t, float cframe)
 
 bool FrameOnMouseSide(char side, float frame, float cframe)
 {
-  /* both sides, so it doesn't matter */
+  /* Both sides, so it doesn't matter. */
   if (side == 'B') {
     return true;
   }
 
-  /* only on the named side */
+  /* Only on the named side. */
   if (side == 'R') {
     return (frame >= cframe);
   }
@@ -557,15 +544,14 @@ bool FrameOnMouseSide(char side, float frame, float cframe)
 
 bool constraints_list_needinv(TransInfo *t, ListBase *list)
 {
-  /* loop through constraints, checking if there's one of the mentioned
-   * constraints needing special crazy-space corrections
-   */
+  /* Loop through constraints, checking if there's one of the mentioned
+   * constraints needing special crazy-space corrections. */
   if (list) {
     LISTBASE_FOREACH (bConstraint *, con, list) {
-      /* only consider constraint if it is enabled, and has influence on result */
+      /* Only consider constraint if it is enabled, and has influence on result. */
       if ((con->flag & (CONSTRAINT_DISABLE | CONSTRAINT_OFF)) == 0 && (con->enforce != 0.0f)) {
-        /* (affirmative) returns for specific constraints here... */
-        /* constraints that require this regardless. */
+        /* Affirmative: returns for specific constraints here. */
+        /* Constraints that require this regardless. */
         if (ELEM(con->type,
                  CONSTRAINT_TYPE_FOLLOWPATH,
                  CONSTRAINT_TYPE_CLAMPTO,
@@ -576,22 +562,24 @@ bool constraints_list_needinv(TransInfo *t, ListBase *list)
           return true;
         }
 
-        /* constraints that require this only under special conditions */
+        /* Constraints that require this only under special conditions. */
         if (con->type == CONSTRAINT_TYPE_CHILDOF) {
           /* ChildOf constraint only works when using all location components, see #42256. */
           bChildOfConstraint *data = (bChildOfConstraint *)con->data;
 
           if ((data->flag & CHILDOF_LOCX) && (data->flag & CHILDOF_LOCY) &&
-              (data->flag & CHILDOF_LOCZ)) {
+              (data->flag & CHILDOF_LOCZ))
+          {
             return true;
           }
         }
         else if (con->type == CONSTRAINT_TYPE_ROTLIKE) {
-          /* CopyRot constraint only does this when rotating, and offset is on */
+          /* CopyRot constraint only does this when rotating, and offset is on. */
           bRotateLikeConstraint *data = (bRotateLikeConstraint *)con->data;
 
           if (ELEM(data->mix_mode, ROTLIKE_MIX_OFFSET, ROTLIKE_MIX_BEFORE) &&
-              ELEM(t->mode, TFM_ROTATION)) {
+              ELEM(t->mode, TFM_ROTATION))
+          {
             return true;
           }
         }
@@ -636,7 +624,7 @@ bool constraints_list_needinv(TransInfo *t, ListBase *list)
     }
   }
 
-  /* no appropriate candidates found */
+  /* No appropriate candidates found. */
   return false;
 }
 
@@ -652,7 +640,7 @@ void special_aftertrans_update(bContext *C, TransInfo *t)
    * with transform's order of freeing (campbell).
    * Order changed, the sequencer stuff should go back in here. */
 
-  /* early out when nothing happened */
+  /* Early out when nothing happened. */
   if (t->data_len_all == 0 || t->mode == TFM_DUMMY) {
     return;
   }
@@ -705,9 +693,7 @@ static int countAndCleanTransDataContainer(TransInfo *t)
     if (tc->data_len == 0) {
       uint index = tc - t->data_container;
       if (index + 1 != t->data_container_len) {
-        SWAP(TransDataContainer,
-             t->data_container[index],
-             t->data_container[t->data_container_len - 1]);
+        std::swap(t->data_container[index], t->data_container[t->data_container_len - 1]);
       }
       t->data_container_len -= 1;
     }
@@ -725,25 +711,26 @@ static int countAndCleanTransDataContainer(TransInfo *t)
 static void init_proportional_edit(TransInfo *t)
 {
   /* NOTE: Proportional editing is not usable in pose mode yet #32444. */
-  if (!ELEM(t->data_type,
-            &TransConvertType_Action,
-            &TransConvertType_Curve,
-            &TransConvertType_Curves,
-            &TransConvertType_Graph,
-            &TransConvertType_GPencil,
-            &TransConvertType_Lattice,
-            &TransConvertType_Mask,
-            &TransConvertType_MBall,
-            &TransConvertType_Mesh,
-            &TransConvertType_MeshEdge,
-            &TransConvertType_MeshSkin,
-            &TransConvertType_MeshUV,
-            &TransConvertType_MeshVertCData,
-            &TransConvertType_Node,
-            &TransConvertType_Object,
-            &TransConvertType_Particle))
+  /* NOTE: This `ELEM` uses more than 16 elements and so has been split. */
+  if (!(ELEM(t->data_type,
+             &TransConvertType_Action,
+             &TransConvertType_Curve,
+             &TransConvertType_Curves,
+             &TransConvertType_Graph,
+             &TransConvertType_GreasePencil,
+             &TransConvertType_Lattice,
+             &TransConvertType_Mask,
+             &TransConvertType_MBall,
+             &TransConvertType_Mesh,
+             &TransConvertType_MeshEdge,
+             &TransConvertType_MeshSkin,
+             &TransConvertType_MeshUV,
+             &TransConvertType_MeshVertCData,
+             &TransConvertType_Node,
+             &TransConvertType_Object) ||
+        ELEM(t->data_type, &TransConvertType_Particle)))
   {
-    /* Disable proportional editing */
+    /* Disable proportional editing. */
     t->options |= CTX_NO_PET;
     t->flag &= ~T_PROP_EDIT_ALL;
     return;
@@ -793,17 +780,14 @@ static void init_proportional_edit(TransInfo *t)
 }
 
 /* For multi object editing. */
-static void init_TransDataContainers(TransInfo *t,
-                                     Object *obact,
-                                     Object **objects,
-                                     uint objects_len)
+static void init_TransDataContainers(TransInfo *t, Object *obact, Span<Object *> objects)
 {
   if (!ELEM(t->data_type,
             &TransConvertType_Pose,
             &TransConvertType_EditArmature,
             &TransConvertType_Curve,
             &TransConvertType_Curves,
-            &TransConvertType_GPencil,
+            &TransConvertType_GreasePencil,
             &TransConvertType_Lattice,
             &TransConvertType_MBall,
             &TransConvertType_Mesh,
@@ -819,33 +803,32 @@ static void init_TransDataContainers(TransInfo *t,
   const eObjectMode object_mode = eObjectMode(obact ? obact->mode : OB_MODE_OBJECT);
   const short object_type = obact ? obact->type : -1;
 
-  if ((object_mode & OB_MODE_EDIT) || (t->data_type == &TransConvertType_GPencil) ||
+  if ((object_mode & OB_MODE_EDIT) || (t->data_type == &TransConvertType_GreasePencil) ||
       ((object_mode & OB_MODE_POSE) && (object_type == OB_ARMATURE)))
   {
     if (t->data_container) {
       MEM_freeN(t->data_container);
     }
 
-    bool free_objects = false;
-    if (objects == nullptr) {
+    Vector<Object *> local_objects;
+    if (objects.is_empty()) {
       ObjectsInModeParams params = {0};
       params.object_mode = object_mode;
       /* Pose transform operates on `ob->pose` so don't skip duplicate object-data. */
       params.no_dup_data = (object_mode & OB_MODE_POSE) == 0;
-      objects = BKE_view_layer_array_from_objects_in_mode_params(
+      local_objects = BKE_view_layer_array_from_objects_in_mode_params(
           t->scene,
           t->view_layer,
           static_cast<const View3D *>((t->spacetype == SPACE_VIEW3D) ? t->view : nullptr),
-          &objects_len,
           &params);
-      free_objects = true;
+      objects = local_objects;
     }
 
     t->data_container = static_cast<TransDataContainer *>(
-        MEM_callocN(sizeof(*t->data_container) * objects_len, __func__));
-    t->data_container_len = objects_len;
+        MEM_callocN(sizeof(*t->data_container) * objects.size(), __func__));
+    t->data_container_len = objects.size();
 
-    for (int i = 0; i < objects_len; i++) {
+    for (int i = 0; i < objects.size(); i++) {
       TransDataContainer *tc = &t->data_container[i];
       if (!(t->flag & T_NO_MIRROR) && (objects[i]->type == OB_MESH)) {
         tc->use_mirror_axis_x = (((Mesh *)objects[i]->data)->symmetry & ME_SYMMETRY_X) != 0;
@@ -855,7 +838,7 @@ static void init_TransDataContainers(TransInfo *t,
 
       if (object_mode & OB_MODE_EDIT) {
         tc->obedit = objects[i];
-        /* Check needed for UVs */
+        /* Check needed for UVs. */
         if ((t->flag & T_2D_EDIT) == 0) {
           tc->use_local_mat = true;
         }
@@ -864,26 +847,22 @@ static void init_TransDataContainers(TransInfo *t,
         tc->poseobj = objects[i];
         tc->use_local_mat = true;
       }
-      else if (t->data_type == &TransConvertType_GPencil) {
+      else if (t->data_type == &TransConvertType_GreasePencil) {
         tc->use_local_mat = true;
       }
 
       if (tc->use_local_mat) {
         BLI_assert((t->flag & T_2D_EDIT) == 0);
-        copy_m4_m4(tc->mat, objects[i]->object_to_world);
+        copy_m4_m4(tc->mat, objects[i]->object_to_world().ptr());
         copy_m3_m4(tc->mat3, tc->mat);
-        /* for non-invertible scale matrices, invert_m4_m4_fallback()
-         * can still provide a valid pivot */
+        /* For non-invertible scale matrices, #invert_m4_m4_fallback()
+         * can still provide a valid pivot. */
         invert_m4_m4_fallback(tc->imat, tc->mat);
         invert_m3_m3(tc->imat3, tc->mat3);
         normalize_m3_m3(tc->mat3_unit, tc->mat3);
       }
       /* Otherwise leave as zero. */
     }
-
-    if (free_objects) {
-      MEM_freeN(objects);
-    }
   }
 }
 
@@ -893,7 +872,7 @@ static TransConvertTypeInfo *convert_type_get(const TransInfo *t, Object **r_obj
   BKE_view_layer_synced_ensure(t->scene, t->view_layer);
   Object *ob = BKE_view_layer_active_object_get(view_layer);
 
-  /* if tests must match recalc_data for correct updates */
+  /* If tests must match recalc_data for correct updates. */
   if (t->options & CTX_CURSOR) {
     if (t->spacetype == SPACE_IMAGE) {
       return &TransConvertType_CursorImage;
@@ -917,7 +896,10 @@ static TransConvertTypeInfo *convert_type_get(const TransInfo *t, Object **r_obj
     return &TransConvertType_MeshEdge;
   }
   if (t->options & CTX_GPENCIL_STROKES) {
-    return &TransConvertType_GPencil;
+    if (t->obedit_type == OB_GREASE_PENCIL) {
+      return &TransConvertType_GreasePencil;
+    }
+    return nullptr;
   }
   if (t->spacetype == SPACE_IMAGE) {
     if (t->options & CTX_MASK) {
@@ -1038,12 +1020,12 @@ void create_trans_data(bContext *C, TransInfo *t)
   t->flag |= eTFlag(t->data_type->flags);
 
   if (ob_armature) {
-    init_TransDataContainers(t, ob_armature, &ob_armature, 1);
+    init_TransDataContainers(t, ob_armature, {ob_armature});
   }
   else {
     BKE_view_layer_synced_ensure(t->scene, t->view_layer);
     Object *ob = BKE_view_layer_active_object_get(t->view_layer);
-    init_TransDataContainers(t, ob, nullptr, 0);
+    init_TransDataContainers(t, ob, {});
   }
 
   if (t->data_type == &TransConvertType_Object) {
@@ -1059,17 +1041,17 @@ void create_trans_data(bContext *C, TransInfo *t)
       t->options |= CTX_OBMODE_XFORM_SKIP_CHILDREN;
     }
     TransConvertType_Object.create_trans_data(C, t);
-    /* Check if we're transforming the camera from the camera */
+    /* Check if we're transforming the camera from the camera. */
     if ((t->spacetype == SPACE_VIEW3D) && (t->region->regiontype == RGN_TYPE_WINDOW)) {
       View3D *v3d = static_cast<View3D *>(t->view);
       RegionView3D *rv3d = static_cast<RegionView3D *>(t->region->regiondata);
       if ((rv3d->persp == RV3D_CAMOB) && v3d->camera) {
-        /* we could have a flag to easily check an object is being transformed */
-        if (v3d->camera->id.tag & LIB_TAG_DOIT) {
+        /* We could have a flag to easily check an object is being transformed. */
+        if (v3d->camera->id.tag & ID_TAG_DOIT) {
           t->options |= CTX_CAMERA;
         }
       }
-      else if (v3d->ob_center && v3d->ob_center->id.tag & LIB_TAG_DOIT) {
+      else if (v3d->ob_center && v3d->ob_center->id.tag & ID_TAG_DOIT) {
         t->options |= CTX_CAMERA;
       }
     }
@@ -1121,8 +1103,8 @@ void transform_convert_clip_mirror_modifier_apply(TransDataContainer *tc)
       if (mmd->mirror_ob) {
         float obinv[4][4];
 
-        invert_m4_m4(obinv, mmd->mirror_ob->object_to_world);
-        mul_m4_m4m4(mtx, obinv, ob->object_to_world);
+        invert_m4_m4(obinv, mmd->mirror_ob->object_to_world().ptr());
+        mul_m4_m4m4(mtx, obinv, ob->object_to_world().ptr());
         invert_m4_m4(imtx, mtx);
       }
 
@@ -1185,71 +1167,73 @@ void animrecord_check_state(TransInfo *t, ID *id)
   ScreenAnimData *sad = static_cast<ScreenAnimData *>((animtimer) ? animtimer->customdata :
                                                                     nullptr);
 
-  /* sanity checks */
+  /* Sanity checks. */
   if (ELEM(nullptr, scene, id, sad)) {
     return;
   }
 
-  /* check if we need a new strip if:
-   * - if animtimer is running
-   * - we're not only keying for available channels
-   * - the option to add new actions for each round is not enabled
+  /* Check if we need a new strip if:
+   * - If `animtimer` is running.
+   * - We're not only keying for available channels.
+   * - The option to add new actions for each round is not enabled.
    */
-  if (IS_AUTOKEY_FLAG(scene, INSERTAVAIL) == 0 &&
-      (scene->toolsettings->autokey_flag & ANIMRECORD_FLAG_WITHNLA))
+  if (blender::animrig::is_keying_flag(scene, AUTOKEY_FLAG_INSERTAVAILABLE) == 0 &&
+      (scene->toolsettings->keying_flag & AUTOKEY_FLAG_LAYERED_RECORD))
   {
-    /* if playback has just looped around,
-     * we need to add a new NLA track+strip to allow a clean pass to occur */
+    /* If playback has just looped around,
+     * we need to add a new NLA track+strip to allow a clean pass to occur. */
     if ((sad) && (sad->flag & ANIMPLAY_FLAG_JUMPED)) {
       AnimData *adt = BKE_animdata_from_id(id);
       const bool is_first = (adt) && (adt->nla_tracks.first == nullptr);
 
-      /* perform push-down manually with some differences
-       * NOTE: BKE_nla_action_pushdown() sync warning...
-       */
+      /* Perform push-down manually with some differences
+       * NOTE: #BKE_nla_action_pushdown() sync warning. */
       if ((adt->action) && !(adt->flag & ADT_NLA_EDIT_ON)) {
-        float astart, aend;
+        /* Only push down if action is more than 1-2 frames long. */
+        const float2 frame_range = adt->action->wrap().get_frame_range_of_keys(true);
+        if (frame_range[1] > frame_range[0] + 2.0f) {
+          /* TODO: call BKE_nla_action_pushdown() instead?  */
 
-        /* only push down if action is more than 1-2 frames long */
-        BKE_action_frame_range_calc(adt->action, true, &astart, &aend);
-        if (aend > astart + 2.0f) {
-          NlaStrip *strip = BKE_nlastack_add_strip(adt, adt->action, ID_IS_OVERRIDE_LIBRARY(id));
+          /* Add a new NLA strip to the track, which references the active action + slot. */
+          NlaStrip *strip = BKE_nlastack_add_strip({*id, *adt}, ID_IS_OVERRIDE_LIBRARY(id));
+          BLI_assert(strip);
+          animrig::nla::assign_action_slot_handle(*strip, adt->slot_handle, *id);
 
-          /* clear reference to action now that we've pushed it onto the stack */
-          id_us_min(&adt->action->id);
-          adt->action = nullptr;
+          /* Clear reference to action now that we've pushed it onto the stack. */
+          const bool unassign_ok = animrig::unassign_action(*id);
+          BLI_assert_msg(
+              unassign_ok,
+              "Expecting un-assigning an action to always work when pushing down an NLA strip");
+          UNUSED_VARS_NDEBUG(unassign_ok);
 
-          /* adjust blending + extend so that they will behave correctly */
+          /* Adjust blending + extend so that they will behave correctly. */
           strip->extendmode = NLASTRIP_EXTEND_NOTHING;
           strip->flag &= ~(NLASTRIP_FLAG_AUTO_BLENDS | NLASTRIP_FLAG_SELECT |
                            NLASTRIP_FLAG_ACTIVE);
 
-          /* copy current "action blending" settings from adt to the strip,
+          /* Copy current "action blending" settings from adt to the strip,
            * as it was keyframed with these settings, so omitting them will
-           * change the effect  [#54766]
-           */
+           * change the effect, see: #54766. */
           if (is_first == false) {
             strip->blendmode = adt->act_blendmode;
             strip->influence = adt->act_influence;
 
             if (adt->act_influence < 1.0f) {
-              /* enable "user-controlled" influence (which will insert a default keyframe)
-               * so that the influence doesn't get lost on the new update
+              /* Enable "user-controlled" influence (which will insert a default keyframe)
+               * so that the influence doesn't get lost on the new update.
                *
                * NOTE: An alternative way would have been to instead hack the influence
                * to not get always get reset to full strength if NLASTRIP_FLAG_USR_INFLUENCE
                * is disabled but auto-blending isn't being used. However, that approach
                * is a bit hacky/hard to discover, and may cause backwards compatibility issues,
-               * so it's better to just do it this way.
-               */
+               * so it's better to just do it this way. */
               strip->flag |= NLASTRIP_FLAG_USR_INFLUENCE;
               BKE_nlastrip_validate_fcurves(strip);
             }
           }
 
-          /* also, adjust the AnimData's action extend mode to be on
-           * 'nothing' so that previous result still play
-           */
+          /* Also, adjust the AnimData's action extend mode to be on
+           * 'nothing' so that previous result still play. */
           adt->act_extendmode = NLASTRIP_EXTEND_NOTHING;
         }
       }
