@@ -10,33 +10,30 @@
 
 #include "MEM_guardedalloc.h"
 
-#include "DNA_gpencil_legacy_types.h"
-
 #include "BLI_math_matrix.h"
+#include "BLI_math_matrix.hh"
 #include "BLI_math_rotation.h"
 #include "BLI_math_vector.h"
 #include "BLI_string.h"
 #include "BLI_task.h"
 
-#include "BKE_context.h"
-#include "BKE_image.h"
-#include "BKE_report.h"
-#include "BKE_unit.h"
+#include "BKE_image.hh"
+#include "BKE_report.hh"
+#include "BKE_unit.hh"
 
-#include "ED_node.hh"
 #include "ED_screen.hh"
 
-#include "WM_api.hh"
-
 #include "UI_interface.hh"
 
-#include "BLT_translation.h"
+#include "BLT_translation.hh"
 
 #include "transform.hh"
 #include "transform_convert.hh"
 #include "transform_mode.hh"
 #include "transform_snap.hh"
 
+using namespace blender;
+
 /* -------------------------------------------------------------------- */
 /** \name Transform (Translate) Custom Data
  * \{ */
@@ -57,6 +54,8 @@ struct TranslateCustomData {
   struct {
     enum eTranslateRotateMode rotate_mode;
   } prev;
+
+  float3 snap_target_grid;
 };
 
 /** \} */
@@ -71,16 +70,16 @@ struct TranslateCustomData {
 struct TransDataArgs_Translate {
   const TransInfo *t;
   const TransDataContainer *tc;
-  float pivot_local[3];
-  float vec[3];
+  float3 snap_source_local;
+  float3 vec;
   enum eTranslateRotateMode rotate_mode;
 };
 
 static void transdata_elem_translate(const TransInfo *t,
                                      const TransDataContainer *tc,
                                      TransData *td,
-                                     const float pivot_local[3],
-                                     const float vec[3],
+                                     const float3 &snap_source_local,
+                                     const float3 &vec,
                                      enum eTranslateRotateMode rotate_mode)
 {
   float rotate_offset[3] = {0};
@@ -96,7 +95,7 @@ static void transdata_elem_translate(const TransInfo *t,
     else {
       BLI_assert(rotate_mode == TRANSLATE_ROTATE_ON);
 
-      const float *original_normal;
+      float3 original_normal;
 
       /* In pose mode, we want to align normals with Y axis of bones. */
       if (t->options & CTX_POSE_BONE) {
@@ -106,10 +105,16 @@ static void transdata_elem_translate(const TransInfo *t,
         original_normal = td->axismtx[2];
       }
 
+      if (t->flag & T_POINTS) {
+        /* Convert to Global Space since #ElementRotation_ex operates with the matrix in global
+         * space. */
+        original_normal = math::transform_direction(float3x3(td->mtx), original_normal);
+      }
+
       rotation_between_vecs_to_mat3(mat, original_normal, t->tsnap.snapNormal);
     }
 
-    ElementRotation_ex(t, tc, td, mat, pivot_local);
+    ElementRotation_ex(t, tc, td, mat, snap_source_local);
 
     if (td->loc) {
       use_rotate_offset = true;
@@ -134,9 +139,9 @@ static void transdata_elem_translate(const TransInfo *t,
 
   if (t->options & CTX_GPENCIL_STROKES) {
     /* Grease pencil multi-frame falloff. */
-    bGPDstroke *gps = (bGPDstroke *)td->extra;
-    if (gps != nullptr) {
-      mul_v3_fl(tvec, td->factor * gps->runtime.multi_frame_falloff);
+    float *gp_falloff = static_cast<float *>(td->extra);
+    if (gp_falloff != nullptr) {
+      mul_v3_fl(tvec, td->factor * *gp_falloff);
     }
     else {
       mul_v3_fl(tvec, td->factor);
@@ -165,7 +170,8 @@ static void transdata_elem_translate_fn(void *__restrict iter_data_v,
   if (td->flag & TD_SKIP) {
     return;
   }
-  transdata_elem_translate(data->t, data->tc, td, data->pivot_local, data->vec, data->rotate_mode);
+  transdata_elem_translate(
+      data->t, data->tc, td, data->snap_source_local, data->vec, data->rotate_mode);
 }
 
 /** \} */
@@ -179,9 +185,8 @@ static void translate_dist_to_str(char *r_str,
                                   const float val,
                                   const UnitSettings *unit)
 {
-  if (unit) {
-    BKE_unit_value_as_string(
-        r_str, r_str_maxncpy, val * unit->scale_length, 4, B_UNIT_LENGTH, unit, false);
+  if (unit && (unit->system != USER_UNIT_NONE)) {
+    BKE_unit_value_as_string_scaled(r_str, r_str_maxncpy, val, 4, B_UNIT_LENGTH, *unit, false);
   }
   else {
     /* Check range to prevent string buffer overflow. */
@@ -196,13 +201,13 @@ static void headerTranslation(TransInfo *t, const float vec[3], char str[UI_MAX_
   char dist_str[NUM_STR_REP_LEN];
   float dist;
 
-  UnitSettings *unit = nullptr;
+  const UnitSettings *unit = nullptr;
   if (!(t->flag & T_2D_EDIT)) {
     unit = &t->scene->unit;
   }
 
   if (hasNumInput(&t->num)) {
-    outputNumInput(&(t->num), dvec_str[0], &t->scene->unit);
+    outputNumInput(&(t->num), dvec_str[0], t->scene->unit);
     dist = len_v3(t->num.val);
   }
   else {
@@ -219,6 +224,10 @@ static void headerTranslation(TransInfo *t, const float vec[3], char str[UI_MAX_
       dvec[0] = val - ival;
     }
 
+    if (t->flag & T_2D_EDIT) {
+      applyAspectRatio(t, dvec);
+    }
+
     if (t->con.mode & CON_APPLY) {
       int i = 0;
       if (t->con.mode & CON_AXIS0) {
@@ -235,10 +244,6 @@ static void headerTranslation(TransInfo *t, const float vec[3], char str[UI_MAX_
       }
     }
 
-    if (t->flag & T_2D_EDIT) {
-      applyAspectRatio(t, dvec);
-    }
-
     dist = len_v3(dvec);
 
     for (int i = 0; i < 3; i++) {
@@ -255,7 +260,7 @@ static void headerTranslation(TransInfo *t, const float vec[3], char str[UI_MAX_
     ofs += BLI_snprintf_rlen(str + ofs,
                              UI_MAX_DRAW_STR - ofs,
                              "%s %s: %s   ",
-                             TIP_("Proportional Size"),
+                             IFACE_("Proportional Size"),
                              t->proptext,
                              prop_str);
   }
@@ -264,7 +269,7 @@ static void headerTranslation(TransInfo *t, const float vec[3], char str[UI_MAX_
     short chainlen = t->settings->autoik_chainlen;
     if (chainlen) {
       ofs += BLI_snprintf_rlen(
-          str + ofs, UI_MAX_DRAW_STR - ofs, TIP_("Auto IK Length: %d"), chainlen);
+          str + ofs, UI_MAX_DRAW_STR - ofs, IFACE_("Auto IK Length: %d"), chainlen);
       ofs += BLI_strncpy_rlen(str + ofs, "   ", UI_MAX_DRAW_STR - ofs);
     }
   }
@@ -301,9 +306,10 @@ static void headerTranslation(TransInfo *t, const float vec[3], char str[UI_MAX_
       SpaceNode *snode = (SpaceNode *)t->area->spacedata.first;
       if (U.uiflag & USER_NODE_AUTO_OFFSET) {
         const char *str_dir = (snode->insert_ofs_dir == SNODE_INSERTOFS_DIR_RIGHT) ?
-                                  TIP_("right") :
-                                  TIP_("left");
-        ofs += BLI_snprintf_rlen(str, UI_MAX_DRAW_STR, TIP_("Auto-offset direction: %s"), str_dir);
+                                  IFACE_("right") :
+                                  IFACE_("left");
+        ofs += BLI_snprintf_rlen(
+            str, UI_MAX_DRAW_STR, IFACE_("Auto-offset direction: %s"), str_dir);
       }
     }
     else {
@@ -336,113 +342,26 @@ static void headerTranslation(TransInfo *t, const float vec[3], char str[UI_MAX_
 /** \name Transform (Translation) Snapping
  * \{ */
 
-static void translate_snap_target_grid_ensure(TransInfo *t)
-{
-  /* Only need to calculate once. */
-  if ((t->tsnap.status & SNAP_TARGET_GRID_FOUND) == 0) {
-    if (t->data_type == &TransConvertType_Cursor3D) {
-      /* Use a fallback when transforming the cursor.
-       * In this case the center is _not_ derived from the cursor which is being transformed. */
-      copy_v3_v3(t->tsnap.snap_target_grid, TRANS_DATA_CONTAINER_FIRST_SINGLE(t)->data->iloc);
-    }
-    else if (t->around == V3D_AROUND_CURSOR) {
-      /* Use a fallback for cursor selection,
-       * this isn't useful as a global center for absolute grid snapping
-       * since its not based on the position of the selection. */
-      tranform_snap_target_median_calc(t, t->tsnap.snap_target_grid);
-    }
-    else {
-      copy_v3_v3(t->tsnap.snap_target_grid, t->center_global);
-    }
-    t->tsnap.status |= SNAP_TARGET_GRID_FOUND;
-  }
-}
-
-static void translate_snap_grid_apply(TransInfo *t,
-                                      const int max_index,
-                                      const float grid_dist[3],
-                                      const float loc[3],
-                                      float r_out[3])
-{
-  BLI_assert(max_index <= 2);
-  translate_snap_target_grid_ensure(t);
-  const float *center_global = t->tsnap.snap_target_grid;
-  const float *asp = t->aspect;
-
-  float in[3];
-  if (t->con.mode & CON_APPLY) {
-    BLI_assert(t->tsnap.target_type == SCE_SNAP_TO_NONE);
-    t->con.applyVec(t, nullptr, nullptr, loc, in);
-  }
-  else {
-    copy_v3_v3(in, loc);
-  }
-
-  for (int i = 0; i <= max_index; i++) {
-    const float iter_fac = grid_dist[i] * asp[i];
-    r_out[i] = iter_fac * roundf((in[i] + center_global[i]) / iter_fac) - center_global[i];
-  }
-}
-
-static bool translate_snap_grid(TransInfo *t, float *val)
-{
-  if (!transform_snap_is_active(t)) {
-    return false;
-  }
-
-  if (!(t->tsnap.mode & SCE_SNAP_TO_GRID) || validSnap(t)) {
-    /* Don't do grid snapping if there is a valid snap point. */
-    return false;
-  }
-
-  /* Don't do grid snapping if not in 3D viewport or UV editor */
-  if (!ELEM(t->spacetype, SPACE_VIEW3D, SPACE_IMAGE)) {
-    return false;
-  }
-
-  if (t->mode != TFM_TRANSLATION) {
-    return false;
-  }
-
-  float grid_dist[3];
-  copy_v3_v3(grid_dist, t->snap_spatial);
-  if (t->modifiers & MOD_PRECISION) {
-    mul_v3_fl(grid_dist, t->snap_spatial_precision);
-  }
-
-  /* Early bailing out if no need to snap */
-  if (is_zero_v3(grid_dist)) {
-    return false;
-  }
-
-  translate_snap_grid_apply(t, t->idx_max, grid_dist, val, val);
-  t->tsnap.target_type = SCE_SNAP_TO_GRID;
-  return true;
-}
-
 static void ApplySnapTranslation(TransInfo *t, float vec[3])
 {
   float point[3];
   getSnapPoint(t, point);
 
-  if (t->spacetype == SPACE_NODE) {
-    char border = t->tsnap.snapNodeBorder;
-    if (border & (NODE_LEFT | NODE_RIGHT)) {
-      vec[0] = point[0] - t->tsnap.snap_source[0];
+  if (t->spacetype == SPACE_SEQ) {
+    if (t->region->regiontype == RGN_TYPE_PREVIEW) {
+      transform_snap_sequencer_image_apply_translate(t, vec);
     }
-    if (border & (NODE_BOTTOM | NODE_TOP)) {
-      vec[1] = point[1] - t->tsnap.snap_source[1];
+    else {
+      transform_snap_sequencer_apply_seqslide(t, vec);
     }
   }
-  else if (t->spacetype == SPACE_SEQ) {
-    transform_snap_sequencer_apply_translate(t, vec);
-  }
   else {
     if (t->spacetype == SPACE_VIEW3D) {
       if (t->options & CTX_PAINT_CURVE) {
         if (ED_view3d_project_float_global(t->region, point, point, V3D_PROJ_TEST_NOP) !=
-            V3D_PROJ_RET_OK) {
-          zero_v3(point); /* no good answer here... */
+            V3D_PROJ_RET_OK)
+        {
+          zero_v3(point); /* No good answer here... */
         }
       }
     }
@@ -450,6 +369,49 @@ static void ApplySnapTranslation(TransInfo *t, float vec[3])
     sub_v3_v3v3(vec, point, t->tsnap.snap_source);
   }
 }
+static void translate_snap_increment_init(const TransInfo *t)
+{
+  if (!(t->tsnap.flag & SCE_SNAP_ABS_GRID)) {
+    return;
+  }
+
+  TranslateCustomData *custom_data = static_cast<TranslateCustomData *>(t->custom.mode.data);
+  if (t->data_type == &TransConvertType_Cursor3D) {
+    /* Use a fallback when transforming the cursor.
+     * In this case the center is _not_ derived from the cursor which is being transformed. */
+    custom_data->snap_target_grid = TRANS_DATA_CONTAINER_FIRST_SINGLE(t)->data->iloc;
+  }
+  else if (t->around == V3D_AROUND_CURSOR) {
+    /* Use a fallback for cursor selection,
+     * this isn't useful as a global center for absolute grid snapping
+     * since its not based on the position of the selection. */
+    tranform_snap_target_median_calc(t, custom_data->snap_target_grid);
+  }
+  else {
+    custom_data->snap_target_grid = t->center_global;
+  }
+}
+
+static bool translate_snap_increment(const TransInfo *t, float *r_val)
+{
+  if (!transform_snap_increment_ex(t, (t->con.mode & CON_APPLY) != 0, r_val)) {
+    return false;
+  }
+
+  if (t->tsnap.flag & SCE_SNAP_ABS_GRID) {
+    TranslateCustomData *custom_data = static_cast<TranslateCustomData *>(t->custom.mode.data);
+
+    float3 absolute_grid_snap_offset = custom_data->snap_target_grid;
+    transform_snap_increment_ex(t, (t->con.mode & CON_APPLY) != 0, absolute_grid_snap_offset);
+    absolute_grid_snap_offset -= custom_data->snap_target_grid;
+    add_v3_v3(r_val, absolute_grid_snap_offset);
+
+    if (t->con.mode & CON_APPLY) {
+      t->con.applyVec(t, nullptr, nullptr, r_val, r_val);
+    }
+  }
+  return true;
+}
 
 /** \} */
 
@@ -485,12 +447,12 @@ static void applyTranslationValue(TransInfo *t, const float vec[3])
   }
 
   FOREACH_TRANS_DATA_CONTAINER (t, tc) {
-    float pivot_local[3];
+    float3 snap_source_local(0);
     if (rotate_mode != TRANSLATE_ROTATE_OFF) {
-      copy_v3_v3(pivot_local, t->tsnap.snap_source);
-      /* The pivot has to be in local-space (see #49494) */
+      snap_source_local = t->tsnap.snap_source;
       if (tc->use_local_mat) {
-        mul_m4_v3(tc->imat, pivot_local);
+        /* The pivot has to be in local-space (see #49494). */
+        snap_source_local = math::transform_point(float4x4(tc->imat), snap_source_local);
       }
     }
 
@@ -500,15 +462,15 @@ static void applyTranslationValue(TransInfo *t, const float vec[3])
         if (td->flag & TD_SKIP) {
           continue;
         }
-        transdata_elem_translate(t, tc, td, pivot_local, vec, rotate_mode);
+        transdata_elem_translate(t, tc, td, snap_source_local, vec, rotate_mode);
       }
     }
     else {
       TransDataArgs_Translate data{};
       data.t = t;
       data.tc = tc;
-      copy_v3_v3(data.pivot_local, pivot_local);
-      copy_v3_v3(data.vec, vec);
+      data.snap_source_local = snap_source_local;
+      data.vec = vec;
       data.rotate_mode = rotate_mode;
 
       TaskParallelSettings settings;
@@ -598,7 +560,6 @@ static void applyTranslation(TransInfo *t)
     }
 
     transform_snap_mixed_apply(t, global_dir);
-    translate_snap_grid(t, global_dir);
 
     if (t->con.mode & CON_APPLY) {
       float in[3];
@@ -608,9 +569,7 @@ static void applyTranslation(TransInfo *t)
 
     float incr_dir[3];
     copy_v3_v3(incr_dir, global_dir);
-    if (!(transform_snap_is_active(t) && validSnap(t)) &&
-        transform_snap_increment_ex(t, (t->con.mode & CON_APPLY) != 0, incr_dir))
-    {
+    if (!(transform_snap_is_active(t) && validSnap(t)) && translate_snap_increment(t, incr_dir)) {
 
       /* Test for mixed snap with grid. */
       float snap_dist_sq = FLT_MAX;
@@ -625,7 +584,7 @@ static void applyTranslation(TransInfo *t)
 
   applyTranslationValue(t, global_dir);
 
-  /* evil hack - redo translation if clipping needed */
+  /* Evil hack - redo translation if clipping needed. */
   if (t->flag & T_CLIP_UV && clip_uv_transform_translation(t, global_dir)) {
     applyTranslationValue(t, global_dir);
 
@@ -653,7 +612,7 @@ static void applyTranslationMatrix(TransInfo *t, float mat_xform[4][4])
 static void initTranslation(TransInfo *t, wmOperator * /*op*/)
 {
   if (t->spacetype == SPACE_ACTION) {
-    /* this space uses time translate */
+    /* This space uses time translate. */
     BKE_report(t->reports,
                RPT_ERROR,
                "Use 'Time_Translate' transform mode instead of 'Translation' mode "
@@ -694,6 +653,8 @@ static void initTranslation(TransInfo *t, wmOperator * /*op*/)
   custom_data->prev.rotate_mode = TRANSLATE_ROTATE_OFF;
   t->custom.mode.data = custom_data;
   t->custom.mode.use_free = true;
+
+  translate_snap_increment_init(t);
 }
 
 /** \} */
